@@ -16,7 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define G_LOG_DOMAIN "Evolution Data Server"
+#define G_LOG_DOMAIN "GtdPluginEds"
 
 #include "gtd-plugin-eds.h"
 #include "gtd-provider-goa.h"
@@ -42,7 +42,6 @@ struct _GtdPluginEds
 {
   GObject                 parent;
 
-  GtkCssProvider         *provider;
   ESourceRegistry        *registry;
 
   /* Providers */
@@ -182,73 +181,72 @@ gtd_plugin_eds_goa_client_finish_cb (GObject      *client,
                                      GAsyncResult *result,
                                      gpointer      user_data)
 {
+  g_autoptr (GError) error = NULL;
   GtdPluginEds *self;
   GoaClient *goa_client;
-  GError *error;
+  GList *accounts;
+  GList *l;
 
   self = GTD_PLUGIN_EDS (user_data);
-  error = NULL;
-
   goa_client = goa_client_new_finish (result, &error);
 
-  if (!error)
-    {
-      GList *accounts;
-      GList *l;
-
-      /* Load each supported GoaAccount into a GtdProviderGoa */
-      accounts = goa_client_get_accounts (goa_client);
-
-      for (l = accounts; l != NULL; l = l->next)
-        {
-          GoaObject *object;
-          GoaAccount *account;
-
-          object = l->data;
-          account = goa_object_get_account (object);
-
-          if (g_strv_contains (supported_accounts, goa_account_get_provider_type (account)))
-            {
-              GtdProviderGoa *provider;
-
-              g_debug ("Creating new provider for account '%s'", goa_account_get_identity (account));
-
-              /* Create the new GOA provider */
-              provider = gtd_provider_goa_new (self->registry, account);
-
-              self->providers = g_list_append (self->providers, provider);
-
-              g_signal_emit_by_name (self, "provider-added", provider);
-            }
-
-          g_object_unref (account);
-        }
-
-      /* Connect GoaClient signals */
-      g_signal_connect (goa_client,
-                        "account-added",
-                        G_CALLBACK (gtd_plugin_eds_goa_account_added_cb),
-                        user_data);
-
-      g_signal_connect (goa_client,
-                        "account-removed",
-                        G_CALLBACK (gtd_plugin_eds_goa_account_removed_cb),
-                        user_data);
-
-      g_list_free_full (accounts, g_object_unref);
-    }
-  else
+  if (error)
     {
       g_warning ("%s: %s: %s",
                  G_STRFUNC,
-                 _("Error loading GNOME Online Accounts"),
+                 "Error loading GNOME Online Accounts",
                  error->message);
 
       gtd_manager_emit_error_message (gtd_manager_get_default (),
                                       _("Error loading GNOME Online Accounts"),
-                                      error->message);
+                                      error->message,
+                                      NULL,
+                                      NULL);
       g_clear_error (&error);
     }
+
+  /* Load each supported GoaAccount into a GtdProviderGoa */
+  accounts = goa_client_get_accounts (goa_client);
+
+  for (l = accounts; l != NULL; l = l->next)
+    {
+      GtdProviderGoa *provider;
+      GoaAccount *account;
+      GoaObject *object;
+
+      object = l->data;
+      account = goa_object_get_account (object);
+
+      if (!g_strv_contains (supported_accounts, goa_account_get_provider_type (account)))
+        {
+          g_object_unref (account);
+          continue;
+        }
+
+      g_debug ("Creating new provider for account '%s'", goa_account_get_identity (account));
+
+      /* Create the new GOA provider */
+      provider = gtd_provider_goa_new (self->registry, account);
+
+      self->providers = g_list_append (self->providers, provider);
+
+      g_signal_emit_by_name (self, "provider-added", provider);
+
+      g_object_unref (account);
+    }
+
+  /* Connect GoaClient signals */
+  g_signal_connect (goa_client,
+                    "account-added",
+                    G_CALLBACK (gtd_plugin_eds_goa_account_added_cb),
+                    user_data);
+
+  g_signal_connect (goa_client,
+                    "account-removed",
+                    G_CALLBACK (gtd_plugin_eds_goa_account_removed_cb),
+                    user_data);
+
+  g_list_free_full (accounts, g_object_unref);
 }
 
 
@@ -270,7 +268,7 @@ gtd_plugin_eds_source_registry_finish_cb (GObject      *source_object,
   if (error)
     {
       g_warning ("%s: %s",
-                 _("Error loading Evolution-Data-Server backend"),
+                 "Error loading Evolution-Data-Server backend",
                  error->message);
 
       g_clear_error (&error);
@@ -338,36 +336,10 @@ gtd_plugin_eds_class_init (GtdPluginEdsClass *klass)
 static void
 gtd_plugin_eds_init (GtdPluginEds *self)
 {
-  GError *error = NULL;
-  GFile* css_file;
-
   /* load the source registry */
   e_source_registry_new (NULL,
                          (GAsyncReadyCallback) gtd_plugin_eds_source_registry_finish_cb,
                          self);
-
-  /* load CSS */
-  self->provider = gtk_css_provider_new ();
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (self->provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION + 1);
-
-  css_file = g_file_new_for_uri ("resource:///org/gnome/todo/theme/eds/Adwaita.css");
-
-  gtk_css_provider_load_from_file (self->provider,
-                                   css_file,
-                                   &error);
-  if (error != NULL)
-   {
-     g_warning ("%s: %s: %s",
-                G_STRFUNC,
-                _("Error loading CSS from resource"),
-                error->message);
-
-     g_error_free (error);
-   }
-
-  g_object_unref (css_file);
 }
 
 /* Empty class_finalize method */
