@@ -18,6 +18,8 @@
 
 #define G_LOG_DOMAIN "GtdProviderEds"
 
+#include "gtd-debug.h"
+#include "gtd-eds-autoptr.h"
 #include "gtd-provider-eds.h"
 #include "gtd-task-eds.h"
 #include "gtd-task-list-eds.h"
@@ -42,25 +44,20 @@ typedef struct
   ESourceRegistry      *source_registry;
   ECredentialsPrompter *credentials_prompter;
 
-  GHashTable           *clients;
+  GCancellable         *cancellable;
 
   gint                  lazy_load_id;
 } GtdProviderEdsPrivate;
 
-/* Auxiliary struct for asyncronous task operations */
-typedef struct _TaskData
-{
-  GtdProviderEds     *provider;
-  gpointer           *data;
-} TaskData;
-
 
 static void          gtd_provider_iface_init                     (GtdProviderInterface *iface);
 
+
 G_DEFINE_TYPE_WITH_CODE (GtdProviderEds, gtd_provider_eds, GTD_TYPE_OBJECT,
                          G_ADD_PRIVATE (GtdProviderEds)
                          G_IMPLEMENT_INTERFACE (GTD_TYPE_PROVIDER, gtd_provider_iface_init))
 
+
 enum
 {
   PROP_0,
@@ -74,37 +71,14 @@ enum
   N_PROPS
 };
 
-static TaskData*
-task_data_new (GtdProviderEds *provider,
-               gpointer       *data)
-{
-  TaskData *tdata;
 
-  tdata = g_new0 (TaskData, 1);
-  tdata->provider = provider;
-  tdata->data = data;
-
-  return tdata;
-}
+/*
+ * Auxiliary methods
+ */
 
 static void
-ensure_offline_sync (GtdProviderEds *self,
-                     ESource        *source)
-{
-  GtdProviderEdsPrivate *priv;
-  ESourceOffline *extension;
-
-  priv = gtd_provider_eds_get_instance_private (self);
-
-  extension = e_source_get_extension (source, E_SOURCE_EXTENSION_OFFLINE);
-  e_source_offline_set_stay_synchronized (extension, TRUE);
-
-  e_source_registry_commit_source (priv->source_registry, source, NULL, NULL, NULL);
-}
-
-static void
-gtd_provider_eds_set_default (GtdProviderEds *self,
-                              GtdTaskList    *list)
+set_default_list (GtdProviderEds *self,
+                  GtdTaskList    *list)
 {
   GtdProviderEdsPrivate *priv;
   GtdManager *manager;
@@ -121,72 +95,36 @@ gtd_provider_eds_set_default (GtdProviderEds *self,
 }
 
 static void
-gtd_provider_eds_fill_task_list (GObject      *client,
-                                 GAsyncResult *result,
-                                 gpointer      user_data)
+ensure_offline_sync (GtdProviderEds *self,
+                     ESource        *source)
 {
-  g_autoptr (GError) error = NULL;
-  GtdProviderEds *self;
-  GtdTaskList *list;
-  TaskData *data = user_data;
-  GSList *component_list;
-  GSList *l;
+  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
+  ESourceOffline *extension;
 
-  g_return_if_fail (GTD_IS_PROVIDER_EDS (data->provider));
+  extension = e_source_get_extension (source, E_SOURCE_EXTENSION_OFFLINE);
+  e_source_offline_set_stay_synchronized (extension, TRUE);
 
-  list = GTD_TASK_LIST (data->data);
-  self = data->provider;
-
-  e_cal_client_get_object_list_as_comps_finish (E_CAL_CLIENT (client),
-                                                result,
-                                                &component_list,
-                                                &error);
-
-  gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
-  g_free (data);
-
-  if (error)
-    {
-      g_warning ("Error fetching tasks from list: %s", error->message);
-
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error fetching tasks from list"),
-                                      error->message,
-                                      NULL,
-                                      NULL);
-      return;
-    }
-
-  for (l = component_list; l != NULL; l = l->next)
-    {
-      GtdTask *task;
-
-      task = gtd_task_eds_new (l->data);
-      gtd_task_set_list (task, list);
-
-      gtd_task_list_save_task (list, task);
-    }
-
-  e_cal_client_free_ecalcomp_slist (component_list);
-
-  /* Emit LIST_ADDED signal */
-  g_signal_emit_by_name (self, "list-added", list);
+  e_source_registry_commit_source (priv->source_registry, source, NULL, NULL, NULL);
 }
 
+
+/*
+ * Callbacks
+ */
+
 static void
-gtd_provider_eds_on_client_connected (GObject      *source_object,
-                                      GAsyncResult *result,
-                                      gpointer      user_data)
+on_client_connected_cb (GObject      *source_object,
+                        GAsyncResult *result,
+                        gpointer      user_data)
 {
+  g_autoptr (ESource) default_source = NULL;
+  g_autoptr (ESource) parent = NULL;
   g_autoptr (GError) error = NULL;
   GtdProviderEdsPrivate *priv;
   GtdProviderEds *self;
   GtdTaskListEds *list;
   ECalClient *client;
-  TaskData *data;
-  ESource *default_source;
   ESource *source;
-  ESource *parent;
 
   self = GTD_PROVIDER_EDS (user_data);
   priv = gtd_provider_eds_get_instance_private (self);
@@ -195,7 +133,7 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
 
   if (error)
     {
-      g_debug ("Failed to connect to task list '%s': %s", e_source_get_uid (source), error->message);
+      g_warning ("Failed to connect to task list '%s': %s", e_source_get_uid (source), error->message);
 
       gtd_manager_emit_error_message (gtd_manager_get_default (),
                                       _("Failed to connect to task list"),
@@ -205,32 +143,17 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
       return;
     }
 
+  ensure_offline_sync (self, source);
+
   /* parent source's display name is list's origin */
   parent = e_source_registry_ref_source (priv->source_registry, e_source_get_parent (source));
 
   /* creates a new task list */
-  list = gtd_task_list_eds_new (GTD_PROVIDER (self), source);
-
-  /* ensure tasklist is cached offline */
-  ensure_offline_sync (self, source);
-
-  /* it's not ready until we fetch the list of tasks from client */
-  gtd_object_set_ready (GTD_OBJECT (list), FALSE);
-
-  /* async data */
-  data = task_data_new (user_data, (gpointer) list);
-
-  /* asyncronously fetch the task list */
-  e_cal_client_get_object_list_as_comps (client,
-                                         "contains? \"any\" \"\"",
-                                         NULL,
-                                         gtd_provider_eds_fill_task_list,
-                                         data);
+  list = gtd_task_list_eds_new (GTD_PROVIDER (self), source, client);
 
   priv->task_lists = g_list_append (priv->task_lists, list);
 
   g_object_set_data (G_OBJECT (source), "task-list", list);
-  g_hash_table_insert (priv->clients, source, client);
 
   /* Check if the current list is the default one */
   default_source = e_source_registry_ref_default_task_list (priv->source_registry);
@@ -238,72 +161,37 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
   if (default_source == source)
     g_object_notify (G_OBJECT (self), "default-task-list");
 
-  g_clear_object (&default_source);
-
-  g_object_unref (parent);
-
   g_debug ("Task list '%s' successfully connected", e_source_get_display_name (source));
 }
 
-typedef struct
+static void
+on_source_added_cb (GtdProviderEds *provider,
+                    ESource        *source)
 {
-  GtdProviderEds *provider;
-  ESource        *source;
-} LoadSourceData;
-
-static gboolean
-gtd_provider_eds_load_source_cb (LoadSourceData *data)
-{
-  GtdProviderEdsPrivate *priv;
-  GtdProviderEds *provider;
-  ESource *source;
-
-  provider = data->provider;
-  source = data->source;
-  priv = gtd_provider_eds_get_instance_private (provider);
-
-  if (e_source_has_extension (source, E_SOURCE_EXTENSION_TASK_LIST) &&
-      !g_hash_table_lookup (priv->clients, source) &&
-      GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->should_load_source (provider, source))
+  /* Don't load the source if it's not a tasklist */
+  if (!e_source_has_extension (source, E_SOURCE_EXTENSION_TASK_LIST) ||
+      !GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->should_load_source (provider, source))
     {
-      e_cal_client_connect (source,
-                            E_CAL_CLIENT_SOURCE_TYPE_TASKS,
-                            10, /* seconds to wait */
-                            NULL,
-                            gtd_provider_eds_on_client_connected,
-                            provider);
+      return;
     }
 
-  g_free (data);
-
-  return G_SOURCE_REMOVE;
-}
-
-static void
-gtd_provider_eds_load_source (GtdProviderEds *provider,
-                              ESource        *source)
-{
-  LoadSourceData *data;
-
-  data = g_new0 (LoadSourceData, 1);
-  data->provider = provider;
-  data->source = source;
-
-  /* HACK: I really don't like to use arbitrary timeouts on
-   * my code, but we have absolutely no guarantees that
-   * ESourceRegistry::source-added was emited to the other
-   * objects before. So Milan Crha told me to add this timeout
-   * and "guarantee" that other objects will receive the
-   * signal.
+  /*
+   * The pop_loading() is actually emited by GtdTaskListEds, after the
+   * ECalClientView sends the :complete signal.
    */
-  g_timeout_add (1000,
-                 (GSourceFunc) gtd_provider_eds_load_source_cb,
-                 data);
+  gtd_object_push_loading (GTD_OBJECT (gtd_manager_get_default ()));
+
+  e_cal_client_connect (source,
+                        E_CAL_CLIENT_SOURCE_TYPE_TASKS,
+                        10, /* seconds to wait */
+                        NULL,
+                        on_client_connected_cb,
+                        provider);
 }
 
 static void
-gtd_provider_eds_remove_source (GtdProviderEds *provider,
-                                ESource        *source)
+on_source_removed_cb (GtdProviderEds *provider,
+                      ESource        *source)
 {
   GtdProviderEdsPrivate *priv;
   GtdTaskList *list;
@@ -313,49 +201,34 @@ gtd_provider_eds_remove_source (GtdProviderEds *provider,
 
   priv->task_lists = g_list_remove (priv->task_lists, list);
 
-  g_hash_table_remove (priv->clients, source);
-
-  /* Since all subclasses will have this signal given that they
+  /*
+   * Since all subclasses will have this signal given that they
    * are all GtdProvider implementations, it's not that bad
    * to let it stay here.
    */
   g_signal_emit_by_name (provider, "list-removed", list);
 }
 
-
-
-
-/************************
- * Credentials prompter *
- ************************/
-
 static void
-gtd_manager__invoke_authentication (GObject      *source_object,
-                                    GAsyncResult *result,
-                                    gpointer      user_data)
+on_authentication_invoked_cb (GObject      *source_object,
+                              GAsyncResult *result,
+                              gpointer      user_data)
 {
   g_autoptr (GError) error = NULL;
   ESource *source;
-  gboolean canceled;
 
   source = E_SOURCE (source_object);
 
-  e_source_invoke_authenticate_finish (source,
-                                       result,
-                                       &error);
+  e_source_invoke_authenticate_finish (source, result, &error);
 
-  canceled = g_error_matches (error,
-                                   G_IO_ERROR,
-                                   G_IO_ERROR_CANCELLED);
-
-  if (!canceled)
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
     g_warning ("Failed to prompt for credentials (%s): %s", e_source_get_uid (source), error->message);
 }
 
 static void
-gtd_provider_local_credentials_prompt_done (GObject      *source_object,
-                                            GAsyncResult *result,
-                                            gpointer      user_data)
+on_credentials_prompt_finished_cb (GObject      *source_object,
+                                   GAsyncResult *result,
+                                   gpointer      user_data)
 {
   ETrustPromptResponse response = E_TRUST_PROMPT_RESPONSE_UNKNOWN;
   ESource *source = E_SOURCE (source_object);
@@ -379,19 +252,19 @@ gtd_provider_local_credentials_prompt_done (GObject      *source_object,
   /* Use NULL credentials to reuse those from the last time. */
   e_source_invoke_authenticate (source,
                                 NULL,
-                                NULL /* cancellable */,
-                                gtd_manager__invoke_authentication,
+                                NULL,
+                                on_authentication_invoked_cb,
                                 NULL);
 }
 
 static void
-gtd_provider_eds_credentials_required (ESourceRegistry          *registry,
-                                       ESource                  *source,
-                                       ESourceCredentialsReason  reason,
-                                       const gchar              *certificate_pem,
-                                       GTlsCertificateFlags      certificate_errors,
-                                       const GError             *error,
-                                       gpointer                  user_data)
+on_eds_credentials_required_cb (ESourceRegistry          *registry,
+                                ESource                  *source,
+                                ESourceCredentialsReason  reason,
+                                const gchar              *certificate_pem,
+                                GTlsCertificateFlags      certificate_errors,
+                                const GError             *error,
+                                gpointer                  user_data)
 {
   GtdProviderEdsPrivate *priv;
   GtdProviderEds *self;
@@ -413,7 +286,7 @@ gtd_provider_eds_credentials_required (ESourceRegistry          *registry,
                                      error ? error->message : NULL,
                                      TRUE, // allow saving sources
                                      NULL, // we won't cancel the operation
-                                     gtd_provider_local_credentials_prompt_done,
+                                     on_credentials_prompt_finished_cb,
                                      NULL);
     }
   else if (error && reason == E_SOURCE_CREDENTIALS_REASON_ERROR)
@@ -425,9 +298,9 @@ gtd_provider_eds_credentials_required (ESourceRegistry          *registry,
 }
 
 static void
-default_tasklist_changed_cb (ESourceRegistry *source_registry,
-                             GParamSpec      *pspec,
-                             GtdProviderEds  *self)
+on_default_tasklist_changed_cb (ESourceRegistry *source_registry,
+                                GParamSpec      *pspec,
+                                GtdProviderEds  *self)
 {
   GtdTaskList *list;
   ESource *default_source;
@@ -445,78 +318,157 @@ out:
   g_clear_object (&default_source);
 }
 
+#define REPORT_ERROR(title,error) \
+G_STMT_START \
+  if (error) \
+    { \
+      g_warning ("%s: %s", title, error->message); \
+      gtd_manager_emit_error_message (gtd_manager_get_default (), title, error->message, NULL, NULL); \
+      GTD_RETURN (); \
+    } \
+G_STMT_END
+
 static void
-gtd_provider_eds_load_registry (GtdProviderEds  *provider)
+on_task_created_cb (ECalClient   *client,
+                    GAsyncResult *result,
+                    GtdTask      *task)
 {
-  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (provider);
   g_autoptr (GError) error = NULL;
-  GList *sources;
-  GList *l;
+  g_autofree gchar *new_uid = NULL;
+  GtdProviderEds *self;
+  GtdTaskList *list;
 
-  priv->credentials_prompter = e_credentials_prompter_new (priv->source_registry);
+  GTD_ENTRY;
 
-  if (error)
-    {
-      g_warning ("%s: %s", "Error loading task manager", error->message);
-      return;
-    }
+  self = GTD_PROVIDER_EDS (gtd_task_get_provider (task));
+  list = gtd_task_get_list (task);
 
-  /* First of all, disable authentication dialog for non-tasklists sources */
-  sources = e_source_registry_list_sources (priv->source_registry, NULL);
+  gtd_object_pop_loading (GTD_OBJECT (self));
+  gtd_object_pop_loading (GTD_OBJECT (task));
 
-  for (l = sources; l != NULL; l = g_list_next (l))
-    {
-      ESource *source = E_SOURCE (l->data);
+  e_cal_client_create_object_finish (client, result, &new_uid, &error);
 
-      /* Mark for skip also currently disabled sources */
-      e_credentials_prompter_set_auto_prompt_disabled_for (priv->credentials_prompter,
-                                                           source,
-                                                           !e_source_has_extension (source, E_SOURCE_EXTENSION_TASK_LIST));
-    }
+  REPORT_ERROR (_("An error occurred while creating a task"), error);
 
-  g_list_free_full (sources, g_object_unref);
+  /* Update the default tasklist */
+  set_default_list (self, list);
 
-  /* Load task list sources */
-  sources = e_source_registry_list_sources (priv->source_registry, E_SOURCE_EXTENSION_TASK_LIST);
+  /*
+   * In the case the task UID changes because of creation proccess,
+   * reapply it to the task.
+   */
+  if (new_uid)
+    gtd_object_set_uid (GTD_OBJECT (task), new_uid);
 
-  for (l = sources; l != NULL; l = l->next)
-    gtd_provider_eds_load_source (provider, l->data);
-
-  g_list_free_full (sources, g_object_unref);
-
-  /* listen to the signals, so new sources don't slip by */
-  g_signal_connect_swapped (priv->source_registry,
-                            "source-added",
-                            G_CALLBACK (gtd_provider_eds_load_source),
-                            provider);
-
-  g_signal_connect_swapped (priv->source_registry,
-                            "source-removed",
-                            G_CALLBACK (gtd_provider_eds_remove_source),
-                            provider);
-
-  g_signal_connect (priv->source_registry,
-                    "credentials-required",
-                    G_CALLBACK (gtd_provider_eds_credentials_required),
-                    provider);
-
-  g_signal_connect (priv->source_registry,
-                    "notify::default-task-list",
-                    G_CALLBACK (default_tasklist_changed_cb),
-                    provider);
-
-  e_credentials_prompter_process_awaiting_credentials (priv->credentials_prompter);
+  GTD_EXIT;
 }
 
 static void
-gtd_provider_eds_set_registry (GtdProviderEds  *provider,
-                               ESourceRegistry *registry)
+on_task_modified_cb (ECalClient   *client,
+                     GAsyncResult *result,
+                     GtdTask      *task)
 {
-  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (provider);
+  g_autoptr (GError) error = NULL;
+  GtdProviderEds *self;
 
-  g_set_object (&priv->source_registry, registry);
+  GTD_ENTRY;
 
-  gtd_provider_eds_load_registry (provider);
+  self = GTD_PROVIDER_EDS (gtd_task_get_provider (task));
+
+  gtd_object_pop_loading (GTD_OBJECT (task));
+  gtd_object_pop_loading (GTD_OBJECT (self));
+
+  e_cal_client_modify_object_finish (client, result, &error);
+
+  REPORT_ERROR (_("An error occurred while modifying a task"), error);
+
+  GTD_EXIT;
+}
+
+static void
+on_task_removed_cb (GObject      *object,
+                    GAsyncResult *result,
+                    gpointer      user_data)
+{
+  g_autoptr (GError) error = NULL;
+  GtdProviderEds *self;
+
+  GTD_ENTRY;
+
+  self = GTD_PROVIDER_EDS (user_data);
+
+  gtd_object_pop_loading (GTD_OBJECT (self));
+
+  e_cal_client_remove_object_finish (E_CAL_CLIENT (object), result, &error);
+
+  REPORT_ERROR (_("An error occurred while removing a task"), error);
+
+  GTD_EXIT;
+}
+
+static void
+on_task_list_created_cb (ESourceRegistry *registry,
+                         GAsyncResult    *result,
+                         GtdProviderEds  *self)
+{
+  g_autoptr (GError) error = NULL;
+
+  GTD_ENTRY;
+
+  gtd_object_pop_loading (GTD_OBJECT (self));
+
+  e_source_registry_commit_source_finish (registry, result, &error);
+
+  REPORT_ERROR (_("An error occurred while creating a task list"), error);
+
+  GTD_EXIT;
+}
+
+static void
+on_task_list_modified_cb (ESourceRegistry *registry,
+                          GAsyncResult    *result,
+                          GtdTaskList     *list)
+{
+  g_autoptr (GError) error = NULL;
+  GtdProviderEds *self;
+
+  GTD_ENTRY;
+
+  self = GTD_PROVIDER_EDS (gtd_task_list_get_provider (list));
+
+  gtd_object_pop_loading (GTD_OBJECT (self));
+  gtd_object_pop_loading (GTD_OBJECT (list));
+
+  e_source_registry_commit_source_finish (registry, result, &error);
+
+  REPORT_ERROR (_("An error occurred while modifying a task list"), error);
+
+  g_signal_emit_by_name (self, "list-changed", list);
+
+  GTD_EXIT;
+}
+
+static void
+on_task_list_removed_cb (ESource      *source,
+                         GAsyncResult *result,
+                         GtdTaskList  *list)
+{
+  g_autoptr (GError) error = NULL;
+  GtdProviderEds *self;
+
+  GTD_ENTRY;
+
+  self = GTD_PROVIDER_EDS (gtd_task_list_get_provider (list));
+
+  gtd_object_pop_loading (GTD_OBJECT (self));
+
+  e_source_remove_finish (source, result, &error);
+
+  REPORT_ERROR (_("An error occurred while modifying a task list"), error);
+
+  g_signal_emit_by_name (self, "list-removed", list);
+
+  GTD_EXIT;
 }
 
 
@@ -566,142 +518,124 @@ gtd_provider_eds_get_icon (GtdProvider *provider)
 }
 
 static void
-gtd_provider_eds_create_task (GtdProvider   *provider,
-                              GtdTask       *task,
-                              GCancellable  *cancellable,
-                              GError       **error)
+gtd_provider_eds_create_task (GtdProvider *provider,
+                              GtdTaskList *list,
+                              const gchar *title,
+                              GDateTime   *due_date)
 {
-  g_autofree gchar *new_uid = NULL;
-  GtdProviderEdsPrivate *priv;
-  GtdProviderEds *self;
   GtdTaskListEds *tasklist;
+  GtdProviderEds *self;
   ECalComponent *component;
   ECalClient *client;
-  ESource *source;
+  GtdTask *new_task;
 
-  g_return_if_fail (GTD_IS_TASK (task));
-  g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
+  g_return_if_fail (GTD_IS_TASK_LIST_EDS (list));
+
+  GTD_ENTRY;
 
   self = GTD_PROVIDER_EDS (provider);
-  priv = gtd_provider_eds_get_instance_private (self);
-  tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
-  source = gtd_task_list_eds_get_source (tasklist);
-  client = g_hash_table_lookup (priv->clients, source);
-  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
+  tasklist = GTD_TASK_LIST_EDS (list);
+  client = gtd_task_list_eds_get_client (tasklist);
+
+  /* Create the new task */
+  component = e_cal_component_new ();
+  e_cal_component_set_new_vtype (component, E_CAL_COMPONENT_TODO);
+
+  new_task = gtd_task_eds_new (component);
+  gtd_task_set_title (new_task, title);
+  gtd_task_set_due_date (new_task, due_date);
+  gtd_task_set_list (new_task, list);
+
+  gtd_task_list_save_task (list, new_task);
 
   /* The task is not ready until we finish the operation */
-  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
+  gtd_object_push_loading (GTD_OBJECT (self));
+  gtd_object_push_loading (GTD_OBJECT (new_task));
 
-  if (!e_cal_client_create_object_sync (client,
-                                        e_cal_component_get_icalcomponent (component),
-                                        &new_uid,
-                                        cancellable,
-                                        error))
-    {
-      gtd_task_list_remove_task (gtd_task_get_list (task), task);
-    }
-  else
-    {
-      /* Update the default tasklist */
-      gtd_provider_eds_set_default (self, gtd_task_get_list (task));
+  e_cal_client_create_object (client,
+                              e_cal_component_get_icalcomponent (component),
+                              NULL,
+                              (GAsyncReadyCallback) on_task_created_cb,
+                              new_task);
 
-      /*
-       * In the case the task UID changes because of creation proccess,
-       * reapply it to the task.
-       */
-      if (new_uid)
-        gtd_object_set_uid (GTD_OBJECT (task), new_uid);
-    }
+  GTD_EXIT;
 }
 
 static void
-gtd_provider_eds_update_task (GtdProvider   *provider,
-                              GtdTask       *task,
-                              GCancellable  *cancellable,
-                              GError       **error)
+gtd_provider_eds_update_task (GtdProvider *provider,
+                              GtdTask     *task)
 {
-  GtdProviderEdsPrivate *priv;
-  GtdProviderEds *self;
   GtdTaskListEds *tasklist;
   ECalComponent *component;
   ECalClient *client;
-  ESource *source;
+
+  GTD_ENTRY;
 
   g_return_if_fail (GTD_IS_TASK (task));
   g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
 
-  self = GTD_PROVIDER_EDS (provider);
-  priv = gtd_provider_eds_get_instance_private (self);
   tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
-  source = gtd_task_list_eds_get_source (tasklist);
-  client = g_hash_table_lookup (priv->clients, source);
+  client = gtd_task_list_eds_get_client (tasklist);
   component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
 
   e_cal_component_commit_sequence (component);
 
   /* The task is not ready until we finish the operation */
-  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
+  gtd_object_push_loading (GTD_OBJECT (task));
+  gtd_object_push_loading (GTD_OBJECT (provider));
 
-  e_cal_client_modify_object_sync (client,
-                                   e_cal_component_get_icalcomponent (component),
-                                   E_CAL_OBJ_MOD_THIS,
-                                   cancellable,
-                                   error);
+  e_cal_client_modify_object (client,
+                              e_cal_component_get_icalcomponent (component),
+                              E_CAL_OBJ_MOD_THIS,
+                              NULL,
+                              (GAsyncReadyCallback) on_task_modified_cb,
+                              task);
 
-  gtd_object_set_ready (GTD_OBJECT (task), TRUE);
+  GTD_EXIT;
 }
 
 static void
-gtd_provider_eds_remove_task (GtdProvider   *provider,
-                              GtdTask       *task,
-                              GCancellable  *cancellable,
-                              GError       **error)
+gtd_provider_eds_remove_task (GtdProvider *provider,
+                              GtdTask     *task)
 {
-
-  GtdProviderEdsPrivate *priv;
-  ECalComponentId *id;
-  GtdProviderEds *self;
+  g_autoptr (ECalComponentId) id = NULL;
   GtdTaskListEds *tasklist;
   ECalComponent *component;
   ECalClient *client;
-  ESource *source;
+
+  GTD_ENTRY;
 
   g_return_if_fail (GTD_IS_TASK (task));
   g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
 
-  self = GTD_PROVIDER_EDS (provider);
-  priv = gtd_provider_eds_get_instance_private (self);
   tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
-  source = gtd_task_list_eds_get_source (tasklist);
-  client = g_hash_table_lookup (priv->clients, source);
+  client = gtd_task_list_eds_get_client (tasklist);
   component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
   id = e_cal_component_get_id (component);
 
-  /* The task is not ready until we finish the operation */
-  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
+  gtd_object_push_loading (GTD_OBJECT (provider));
 
-  e_cal_client_remove_object_sync (client,
-                                   id->uid,
-                                   id->rid,
-                                   E_CAL_OBJ_MOD_THIS,
-                                   cancellable,
-                                   error);
+  e_cal_client_remove_object (client,
+                              id->uid,
+                              id->rid,
+                              E_CAL_OBJ_MOD_THIS,
+                              NULL,
+                              (GAsyncReadyCallback) on_task_removed_cb,
+                              provider);
 
-  gtd_object_set_ready (GTD_OBJECT (task), TRUE);
-
-  e_cal_component_free_id (id);
+  GTD_EXIT;
 }
 
 static void
-gtd_provider_eds_create_task_list (GtdProvider   *provider,
-                                   GtdTaskList   *list,
-                                   GCancellable  *cancellable,
-                                   GError       **error)
+gtd_provider_eds_create_task_list (GtdProvider *provider,
+                                   const gchar *name)
 {
   GtdProviderEdsPrivate *priv;
   GtdProviderEds *self;
   ESource *source;
 
+  GTD_ENTRY;
+
   self = GTD_PROVIDER_EDS (provider);
   priv = gtd_provider_eds_get_instance_private (self);
   source = NULL;
@@ -713,66 +647,68 @@ gtd_provider_eds_create_task_list (GtdProvider   *provider,
   if (!source)
     return;
 
+  gtd_object_push_loading (GTD_OBJECT (provider));
+
   /* EDS properties */
-  e_source_set_display_name (source, gtd_task_list_get_name (list));
+  e_source_set_display_name (source, name);
 
-  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
+  e_source_registry_commit_source (priv->source_registry,
+                                   source,
+                                   NULL,
+                                   (GAsyncReadyCallback) on_task_list_created_cb,
+                                   provider);
 
-  e_source_registry_commit_source_sync (priv->source_registry,
-                                        source,
-                                        cancellable,
-                                        error);
-
-  gtd_object_set_ready (GTD_OBJECT (list), TRUE);
+  GTD_EXIT;
 }
 
 static void
-gtd_provider_eds_update_task_list (GtdProvider   *provider,
-                                   GtdTaskList   *list,
-                                   GCancellable  *cancellable,
-                                   GError       **error)
+gtd_provider_eds_update_task_list (GtdProvider *provider,
+                                   GtdTaskList *list)
 {
   GtdProviderEdsPrivate *priv;
   ESource *source;
 
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
-  g_return_if_fail (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)));
+  GTD_ENTRY;
+
+  g_assert (GTD_IS_TASK_LIST_EDS (list));
+  g_assert (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)) != NULL);
 
   priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (provider));
   source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
 
-  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
-  e_source_registry_commit_source_sync (priv->source_registry,
-                                        source,
-                                        cancellable,
-                                        error);
+  gtd_object_push_loading (GTD_OBJECT (provider));
+  gtd_object_push_loading (GTD_OBJECT (list));
 
-  gtd_object_set_ready (GTD_OBJECT (provider), TRUE);
+  e_source_registry_commit_source (priv->source_registry,
+                                   source,
+                                   NULL,
+                                   (GAsyncReadyCallback) on_task_list_modified_cb,
+                                   list);
 
-  g_signal_emit_by_name (provider, "list-changed", list);
+  GTD_EXIT;
 }
 
 static void
-gtd_provider_eds_remove_task_list (GtdProvider   *provider,
-                                   GtdTaskList   *list,
-                                   GCancellable  *cancellable,
-                                   GError       **error)
+gtd_provider_eds_remove_task_list (GtdProvider *provider,
+                                   GtdTaskList *list)
 {
   ESource *source;
 
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
-  g_return_if_fail (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)));
+  GTD_ENTRY;
+
+  g_assert (GTD_IS_TASK_LIST_EDS (list));
+  g_assert (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)) != NULL);
 
-  error = NULL;
   source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
 
-  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
+  gtd_object_push_loading (GTD_OBJECT (provider));
 
-  e_source_remove_sync (source, cancellable, error);
+  e_source_remove (source,
+                   NULL,
+                   (GAsyncReadyCallback) on_task_list_removed_cb,
+                   list);
 
-  gtd_object_set_ready (GTD_OBJECT (provider), TRUE);
-
-  g_signal_emit_by_name (provider, "list-removed", list);
+  GTD_EXIT;
 }
 
 static GList*
@@ -812,30 +748,16 @@ gtd_provider_eds_set_default_task_list (GtdProvider *provider,
   GtdProviderEdsPrivate *priv;
   ESource *source;
 
+  g_assert (GTD_IS_TASK_LIST_EDS (list));
+
   priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (provider));
-  source = g_object_get_data (G_OBJECT (list), "task-list");
+  source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
 
   e_source_registry_set_default_task_list (priv->source_registry, source);
 
   g_object_notify (G_OBJECT (provider), "default-task-list");
 }
 
-static GtdTask*
-gtd_provider_eds_generate_task (GtdProvider *self)
-{
-  ECalComponent *component;
-  GtdTask *task;
-
-  component = e_cal_component_new ();
-  e_cal_component_set_new_vtype (component, E_CAL_COMPONENT_TODO);
-
-  task = gtd_task_eds_new (component);
-
-  g_object_unref (component);
-
-  return task;
-}
-
 static void
 gtd_provider_iface_init (GtdProviderInterface *iface)
 {
@@ -853,7 +775,6 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
   iface->get_task_lists = gtd_provider_eds_get_task_lists;
   iface->get_default_task_list = gtd_provider_eds_get_default_task_list;
   iface->set_default_task_list = gtd_provider_eds_set_default_task_list;
-  iface->generate_task = gtd_provider_eds_generate_task;
 }
 
 
@@ -867,15 +788,83 @@ gtd_provider_eds_finalize (GObject *object)
   GtdProviderEds *self = (GtdProviderEds *)object;
   GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
 
-  g_signal_handlers_disconnect_by_func (priv->source_registry, default_tasklist_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (priv->source_registry, on_default_tasklist_changed_cb, self);
 
-  g_clear_pointer (&priv->clients, g_hash_table_destroy);
+  g_cancellable_cancel (priv->cancellable);
+
+  g_clear_object (&priv->cancellable);
   g_clear_object (&priv->credentials_prompter);
   g_clear_object (&priv->source_registry);
 
   G_OBJECT_CLASS (gtd_provider_eds_parent_class)->finalize (object);
 }
 
+static void
+gtd_provider_eds_constructed (GObject *object)
+{
+  GtdProviderEdsPrivate *priv;
+  GtdProviderEds *self;
+  g_autoptr (GError) error = NULL;
+  GList *sources;
+  GList *l;
+
+  self = GTD_PROVIDER_EDS (object);
+  priv = gtd_provider_eds_get_instance_private (self);
+  priv->credentials_prompter = e_credentials_prompter_new (priv->source_registry);
+
+  if (error)
+    {
+      g_warning ("%s: %s", "Error loading task manager", error->message);
+      return;
+    }
+
+  /* First of all, disable authentication dialog for non-tasklists sources */
+  sources = e_source_registry_list_sources (priv->source_registry, NULL);
+
+  for (l = sources; l != NULL; l = g_list_next (l))
+    {
+      ESource *source = E_SOURCE (l->data);
+
+      /* Mark for skip also currently disabled sources */
+      e_credentials_prompter_set_auto_prompt_disabled_for (priv->credentials_prompter,
+                                                           source,
+                                                           !e_source_has_extension (source, E_SOURCE_EXTENSION_TASK_LIST));
+    }
+
+  g_list_free_full (sources, g_object_unref);
+
+  /* Load task list sources */
+  sources = e_source_registry_list_sources (priv->source_registry, E_SOURCE_EXTENSION_TASK_LIST);
+
+  for (l = sources; l != NULL; l = l->next)
+    on_source_added_cb (self, l->data);
+
+  g_list_free_full (sources, g_object_unref);
+
+  /* listen to the signals, so new sources don't slip by */
+  g_signal_connect_swapped (priv->source_registry,
+                            "source-added",
+                            G_CALLBACK (on_source_added_cb),
+                            self);
+
+  g_signal_connect_swapped (priv->source_registry,
+                            "source-removed",
+                            G_CALLBACK (on_source_removed_cb),
+                            self);
+
+  g_signal_connect (priv->source_registry,
+                    "credentials-required",
+                    G_CALLBACK (on_eds_credentials_required_cb),
+                    self);
+
+  g_signal_connect (priv->source_registry,
+                    "notify::default-task-list",
+                    G_CALLBACK (on_default_tasklist_changed_cb),
+                    self);
+
+  e_credentials_prompter_process_awaiting_credentials (priv->credentials_prompter);
+}
+
 static void
 gtd_provider_eds_get_property (GObject    *object,
                                guint       prop_id,
@@ -928,6 +917,7 @@ gtd_provider_eds_set_property (GObject      *object,
                                GParamSpec   *pspec)
 {
   GtdProviderEds *self = GTD_PROVIDER_EDS (object);
+  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
 
   switch (prop_id)
     {
@@ -936,7 +926,8 @@ gtd_provider_eds_set_property (GObject      *object,
       break;
 
     case PROP_REGISTRY:
-      gtd_provider_eds_set_registry (self, g_value_get_object (value));
+      if (g_set_object (&priv->source_registry, g_value_get_object (value)))
+        g_object_notify (object, "registry");
       break;
 
     default:
@@ -950,6 +941,7 @@ gtd_provider_eds_class_init (GtdProviderEdsClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = gtd_provider_eds_finalize;
+  object_class->constructed = gtd_provider_eds_constructed;
   object_class->get_property = gtd_provider_eds_get_property;
   object_class->set_property = gtd_provider_eds_set_property;
 
@@ -966,7 +958,7 @@ gtd_provider_eds_class_init (GtdProviderEdsClass *klass)
                                                         "Source registry",
                                                         "The EDS source registry object",
                                                         E_TYPE_SOURCE_REGISTRY,
-                                                        G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -974,14 +966,9 @@ gtd_provider_eds_init (GtdProviderEds *self)
 {
   GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
 
-  /* While it's not ready, we don't load tasklists */
-  gtd_object_set_ready (GTD_OBJECT (self), FALSE);
+  priv->cancellable = g_cancellable_new ();
 
-  /* hash table */
-  priv->clients = g_hash_table_new_full ((GHashFunc) e_source_hash,
-                                         (GEqualFunc) e_source_equal,
-                                         g_object_unref,
-                                         g_object_unref);
+  gtd_object_push_loading (GTD_OBJECT (self));
 }
 
 GtdProviderEds*
