@@ -19,10 +19,12 @@
 #ifndef GTD_PROVIDER_GOA_H
 #define GTD_PROVIDER_GOA_H
 
+#define GOA_API_IS_SUBJECT_TO_CHANGE 1
+
+#include "gnome-todo.h"
 #include "gtd-provider-eds.h"
 
 #include <glib.h>
-#include <gnome-todo.h>
 #include <goa/goa.h>
 
 G_BEGIN_DECLS
