@@ -75,31 +75,12 @@ gtd_provider_local_create_source (GtdProviderEds *provider)
 {
   ESourceExtension *extension;
   ESource *source;
-  GError *error;
-
-  error = NULL;
 
   /* Create the source */
-  source = e_source_new (NULL,
-                         NULL,
-                         &error);
+  source = e_source_new (NULL, NULL, NULL);
 
-  if (error)
-    {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 "Error creating new task list",
-                 error->message);
-
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error creating new task list"),
-                                      error->message,
-                                      NULL,
-                                      NULL);
-
-      g_clear_error (&error);
-      return NULL;
-    }
+  if (!source)
+    return NULL;
 
   /* Make it a local source */
   extension = e_source_get_extension (source, E_SOURCE_EXTENSION_TASK_LIST);
@@ -150,8 +131,6 @@ gtd_provider_local_class_init (GtdProviderLocalClass *klass)
 static void
 gtd_provider_local_init (GtdProviderLocal *self)
 {
-  gtd_object_set_ready (GTD_OBJECT (self), TRUE);
-
   self->icon = G_ICON (g_themed_icon_new_with_default_fallbacks ("computer-symbolic"));
 }
 
