@@ -261,6 +261,13 @@ gtd_task_eds_get_creation_date (GtdTask *task)
   return dt;
 }
 
+static void
+gtd_task_eds_set_creation_date (GtdTask   *task,
+                                GDateTime *dt)
+{
+  g_assert_not_reached ();
+}
+
 static const gchar*
 gtd_task_eds_get_description (GtdTask *task)
 {
@@ -481,6 +488,7 @@ gtd_task_eds_subtask_removed (GtdTask *task,
   icalcomponent_remove_property (ical_comp, property);
 }
 
+
 /*
  * GObject overrides
  */
@@ -551,6 +559,7 @@ gtd_task_eds_class_init (GtdTaskEdsClass *klass)
   task_class->get_complete = gtd_task_eds_get_complete;
   task_class->set_complete = gtd_task_eds_set_complete;
   task_class->get_creation_date = gtd_task_eds_get_creation_date;
+  task_class->set_creation_date = gtd_task_eds_set_creation_date;
   task_class->get_description = gtd_task_eds_get_description;
   task_class->set_description = gtd_task_eds_set_description;
   task_class->get_due_date = gtd_task_eds_get_due_date;
@@ -569,7 +578,7 @@ gtd_task_eds_class_init (GtdTaskEdsClass *klass)
                                                     "Component",
                                                     "Component",
                                                     E_TYPE_CAL_COMPONENT,
-                                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
+                                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, N_PROPS, properties);
 }
@@ -594,3 +603,28 @@ gtd_task_eds_get_component (GtdTaskEds *self)
 
   return self->component;
 }
+
+void
+gtd_task_eds_set_component (GtdTaskEds    *self,
+                            ECalComponent *component)
+{
+  GObject *object;
+
+  g_return_if_fail (GTD_IS_TASK_EDS (self));
+  g_return_if_fail (E_IS_CAL_COMPONENT (component));
+
+  if (!g_set_object (&self->component, component))
+    return;
+
+  object = G_OBJECT (self);
+
+  setup_description (self);
+
+  g_object_notify (object, "complete");
+  g_object_notify (object, "creation-date");
+  g_object_notify (object, "description");
+  g_object_notify (object, "due-date");
+  g_object_notify (object, "priority");
+  g_object_notify (object, "title");
+  g_object_notify_by_pspec (object, properties[PROP_COMPONENT]);
+}
