@@ -19,8 +19,9 @@
 #ifndef GTD_TASK_LIST_EDS_H
 #define GTD_TASK_LIST_EDS_H
 
+#include "gnome-todo.h"
+
 #include <glib-object.h>
-#include <gnome-todo.h>
 #include <libecal/libecal.h>
 
 G_BEGIN_DECLS
@@ -30,13 +31,16 @@ G_BEGIN_DECLS
 G_DECLARE_FINAL_TYPE (GtdTaskListEds, gtd_task_list_eds, GTD, TASK_LIST_EDS, GtdTaskList)
 
 GtdTaskListEds*      gtd_task_list_eds_new                       (GtdProvider        *provider,
-                                                                  ESource            *source);
+                                                                  ESource            *source,
+                                                                  ECalClient         *client);
 
 ESource*             gtd_task_list_eds_get_source                (GtdTaskListEds     *list);
 
 void                 gtd_task_list_eds_set_source                (GtdTaskListEds     *list,
                                                                   ESource            *source);
 
+ECalClient*          gtd_task_list_eds_get_client                (GtdTaskListEds     *self);
+
 G_END_DECLS
 
 #endif /* GTD_TASK_LIST_EDS_H */
