@@ -16,10 +16,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GtdPanelToday"
+
+#include "gnome-todo.h"
 #include "gtd-panel-today.h"
 
 #include <glib/gi18n.h>
-#include <gnome-todo.h>
 
 struct _GtdPanelToday
 {
