@@ -83,19 +83,14 @@ print_task (GString *output,
 {
   GtdTaskList *list;
   GDateTime *dt;
-  const gchar *list_name;
-  const gchar *title;
   gint priority;
   gboolean is_complete;
 
   is_complete = gtd_task_get_complete (task);
-  title = gtd_task_get_title (task);
   priority = gtd_task_get_priority (task);
   dt = gtd_task_get_due_date (task);
   list = gtd_task_get_list (task);
 
-  list_name = gtd_task_list_get_name (list);
-
   if (is_complete)
     g_string_append (output, "x ");
 
@@ -109,7 +104,10 @@ print_task (GString *output,
         g_string_append (output, "(A) ");
     }
 
-  g_string_append_printf (output, "%s @%s", title, list_name);
+  g_string_append_printf (output,
+                          "%s @%s",
+                          gtd_task_get_title (task),
+                          gtd_task_list_get_name (list));
 
   if (dt)
     {
@@ -133,27 +131,36 @@ update_source (GtdProviderTodoTxt *self)
 
   contents = g_string_new ("");
 
+  /* Save the tasks first */
+  for (i = 0; i < self->cache->len; i++)
+    {
+      g_autoptr (GList) tasks = NULL;
+      g_autoptr (GList) l = NULL;
+
+      list = g_ptr_array_index (self->cache, i);
+      tasks = gtd_task_list_get_tasks (list);
+
+      /* And now each task */
+      for (l = tasks; l; l = l->next)
+        print_task (contents, l->data);
+    }
+
+  /* Then the task lists */
   for (i = 0; i < self->cache->len; i++)
     {
       g_autofree gchar *color_str = NULL;
       g_autoptr (GdkRGBA) color = NULL;
-      GList *tasks, *l;
 
       list = g_ptr_array_index (self->cache, i);
 
-      tasks = gtd_task_list_get_tasks (list);
-
       /* Print the list as the first line */
       color = gtd_task_list_get_color (list);
       color_str = gdk_rgba_to_string (color);
 
-      g_string_append_printf (contents, "@%s color:%s\n",
+      g_string_append_printf (contents,
+                              "h:1 @%s color:%s\n",
                               gtd_task_list_get_name (list),
                               color_str);
-
-      /* And now each task */
-      for (l = tasks; l != NULL; l = l->next)
-        print_task (contents, l->data);
     }
 
   output_path = g_file_get_path (self->source_file);
@@ -409,60 +416,68 @@ gtd_provider_todo_txt_get_icon (GtdProvider *provider)
 }
 
 static void
-gtd_provider_todo_txt_create_task (GtdProvider   *provider,
-                                   GtdTask       *task,
-                                   GCancellable  *cancellable,
-                                   GError       **error)
-{
-  update_source (GTD_PROVIDER_TODO_TXT (provider));
-}
-
-static void
-gtd_provider_todo_txt_update_task (GtdProvider   *provider,
-                                   GtdTask       *task,
-                                   GCancellable  *cancellable,
-                                   GError       **error)
-{
-  update_source (GTD_PROVIDER_TODO_TXT (provider));
-}
-
-static void
-gtd_provider_todo_txt_remove_task (GtdProvider   *provider,
-                                   GtdTask       *task,
-                                   GCancellable  *cancellable,
-                                   GError       **error)
-{
-  update_source (GTD_PROVIDER_TODO_TXT (provider));
-}
-
-static void
-gtd_provider_todo_txt_create_task_list (GtdProvider   *provider,
-                                        GtdTaskList   *list,
-                                        GCancellable  *cancellable,
-                                        GError       **error)
+gtd_provider_todo_txt_create_task (GtdProvider *provider,
+                                   GtdTaskList *list,
+                                   const gchar *title,
+                                   GDateTime   *due_date)
 {
   GtdProviderTodoTxt *self;
-  gchar *name;
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
+  g_autoptr (GtdTask) new_task = NULL;
 
   self = GTD_PROVIDER_TODO_TXT (provider);
-  name = g_strdup (gtd_task_list_get_name (list));
-  gtd_task_list_set_is_removable (list, TRUE);
 
-  self->task_lists = g_list_append (self->task_lists, list);
-  g_ptr_array_add (self->cache, list);
-  g_hash_table_insert (self->lists, name, list);
+  /* Create the new task */
+  new_task = gtd_provider_todo_txt_generate_task (self);
+  gtd_task_set_due_date (new_task, due_date);
+  gtd_task_set_list (new_task, list);
+  gtd_task_set_title (new_task, title);
+
+  gtd_task_list_save_task (list, new_task);
+
+  update_source (GTD_PROVIDER_TODO_TXT (provider));
+}
+
+static void
+gtd_provider_todo_txt_update_task (GtdProvider *provider,
+                                   GtdTask     *task)
+{
+  update_source (GTD_PROVIDER_TODO_TXT (provider));
+}
+
+static void
+gtd_provider_todo_txt_remove_task (GtdProvider *provider,
+                                   GtdTask     *task)
+{
+  gtd_task_list_remove_task (gtd_task_get_list (task), task);
+
+  update_source (GTD_PROVIDER_TODO_TXT (provider));
+}
+
+static void
+gtd_provider_todo_txt_create_task_list (GtdProvider *provider,
+                                        const gchar *name)
+{
+  GtdProviderTodoTxt *self;
+  GtdTaskList *new_list;
+
+  self = GTD_PROVIDER_TODO_TXT (provider);
+
+  new_list = gtd_task_list_new (provider);
+  gtd_task_list_set_name (new_list, name);
+  gtd_task_list_set_is_removable (new_list, TRUE);
+
+  self->task_lists = g_list_append (self->task_lists, new_list);
+  g_ptr_array_add (self->cache, new_list);
+  g_hash_table_insert (self->lists, g_strdup (name), new_list);
 
   update_source (self);
 
-  g_signal_emit_by_name (provider, "list-added", list);
+  g_signal_emit_by_name (provider, "list-added", new_list);
 }
 
 static void
-gtd_provider_todo_txt_update_task_list (GtdProvider   *provider,
-                                        GtdTaskList   *list,
-                                        GCancellable  *cancellable,
-                                        GError       **error)
+gtd_provider_todo_txt_update_task_list (GtdProvider *provider,
+                                        GtdTaskList *list)
 {
   GtdProviderTodoTxt *self;
 
@@ -476,10 +491,8 @@ gtd_provider_todo_txt_update_task_list (GtdProvider   *provider,
 }
 
 static void
-gtd_provider_todo_txt_remove_task_list (GtdProvider   *provider,
-                                        GtdTaskList   *list,
-                                        GCancellable  *cancellable,
-                                        GError       **error)
+gtd_provider_todo_txt_remove_task_list (GtdProvider *provider,
+                                        GtdTaskList *list)
 {
   GtdProviderTodoTxt *self = GTD_PROVIDER_TODO_TXT (provider);
 
@@ -512,17 +525,6 @@ gtd_provider_todo_txt_set_default_task_list (GtdProvider *provider,
   /* FIXME: implement me */
 }
 
-static GtdTask*
-gtd_provider_todo_txt_generate_task (GtdProvider *provider)
-{
-  GtdProviderTodoTxt *self = (GtdProviderTodoTxt *) provider;
-  g_autofree gchar *uid = NULL;
-
-  uid = g_strdup_printf ("%ld", self->task_counter);
-
-  return g_object_new (GTD_TYPE_TASK, "uid", uid, NULL);
-}
-
 static void
 gtd_provider_iface_init (GtdProviderInterface *iface)
 {
@@ -540,7 +542,6 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
   iface->get_task_lists = gtd_provider_todo_txt_get_task_lists;
   iface->get_default_task_list = gtd_provider_todo_txt_get_default_task_list;
   iface->set_default_task_list = gtd_provider_todo_txt_set_default_task_list;
-  iface->generate_task = gtd_provider_todo_txt_generate_task;
 }
 
 
@@ -650,8 +651,6 @@ gtd_provider_todo_txt_class_init (GtdProviderTodoTxtClass *klass)
 static void
 gtd_provider_todo_txt_init (GtdProviderTodoTxt *self)
 {
-  gtd_object_set_ready (GTD_OBJECT (self), TRUE);
-
   self->lists = g_hash_table_new (g_str_hash, g_str_equal);
   self->tasks = g_hash_table_new (g_str_hash, g_str_equal);
   self->cache = g_ptr_array_new ();
@@ -667,3 +666,14 @@ gtd_provider_todo_txt_new (GFile *source_file)
                        "source", source_file,
                        NULL);
 }
+
+GtdTask*
+gtd_provider_todo_txt_generate_task (GtdProviderTodoTxt *self)
+{
+  g_autofree gchar *uid = NULL;
+
+  g_return_val_if_fail (GTD_IS_PROVIDER_TODO_TXT (self), NULL);
+  uid = g_uuid_string_random ();
+
+  return g_object_new (GTD_TYPE_TASK, "uid", uid, NULL);
+}
