@@ -96,16 +96,6 @@ static guint signals[NUM_SIGNALS] = { 0, };
  * Auxiliary methods
  */
 
-static void
-reset_cancellable_if_cancelled (GtdManager *self)
-{
-  if (!g_cancellable_is_cancelled (self->cancellable))
-    return;
-
-  g_clear_object (&self->cancellable);
-  self->cancellable = g_cancellable_new ();
-}
-
 static void
 check_provider_is_default (GtdManager  *self,
                            GtdProvider *provider)
@@ -118,22 +108,6 @@ check_provider_is_default (GtdManager  *self,
     gtd_manager_set_default_provider (self, provider);
 }
 
-static void
-emit_show_error_message (GtdManager                *self,
-                         const gchar               *primary_text,
-                         const gchar               *secondary_text,
-                         GtdNotificationActionFunc  action,
-                         gpointer                   user_data)
-{
-  g_signal_emit (self,
-                 signals[SHOW_ERROR_MESSAGE],
-                 0,
-                 primary_text,
-                 secondary_text,
-                 action,
-                 user_data);
-}
-
 
 /*
  * Callbacks
@@ -617,190 +591,6 @@ gtd_manager_new (void)
   return g_object_new (GTD_TYPE_MANAGER, NULL);
 }
 
-/**
- * gtd_manager_create_task:
- * @manager: a #GtdManager
- * @task: a #GtdTask
- *
- * Ask for @task's parent list source to create @task.
- */
-void
-gtd_manager_create_task (GtdManager *self,
-                         GtdTask    *task)
-{
-  g_autoptr (GError) error = NULL;
-  GtdTaskList *list;
-  GtdProvider *provider;
-
-  g_return_if_fail (GTD_IS_MANAGER (self));
-  g_return_if_fail (GTD_IS_TASK (task));
-
-  list = gtd_task_get_list (task);
-  provider = gtd_task_list_get_provider (list);
-
-  gtd_provider_create_task (provider, task, self->cancellable, &error);
-
-  if (error)
-    {
-      g_warning ("Error creating task: %s", error->message);
-      reset_cancellable_if_cancelled (self);
-    }
-}
-
-/**
- * gtd_manager_remove_task:
- * @manager: a #GtdManager
- * @task: a #GtdTask
- *
- * Ask for @task's parent list source to remove @task.
- */
-void
-gtd_manager_remove_task (GtdManager *self,
-                         GtdTask    *task)
-{
-  g_autoptr (GError) error = NULL;
-  GtdTaskList *list;
-  GtdProvider *provider;
-
-  g_return_if_fail (GTD_IS_MANAGER (self));
-  g_return_if_fail (GTD_IS_TASK (task));
-
-  list = gtd_task_get_list (task);
-  provider = gtd_task_list_get_provider (list);
-
-  gtd_provider_remove_task (provider, task, self->cancellable, &error);
-
-  if (error)
-    {
-      g_warning ("Error removing task: %s", error->message);
-      reset_cancellable_if_cancelled (self);
-    }
-}
-
-/**
- * gtd_manager_update_task:
- * @manager: a #GtdManager
- * @task: a #GtdTask
- *
- * Ask for @task's parent list source to update @task.
- */
-void
-gtd_manager_update_task (GtdManager *self,
-                         GtdTask    *task)
-{
-  g_autoptr (GError) error = NULL;
-  GtdTaskList *list;
-  GtdProvider *provider;
-
-  g_return_if_fail (GTD_IS_MANAGER (self));
-  g_return_if_fail (GTD_IS_TASK (task));
-
-  list = gtd_task_get_list (task);
-
-  /* Task does not have parent list, nothing we can do */
-  if (!list)
-      return;
-
-  provider = gtd_task_list_get_provider (list);
-
-  gtd_provider_update_task (provider, task, self->cancellable, &error);
-
-  if (error)
-    {
-      g_warning ("Error updating task: %s", error->message);
-      reset_cancellable_if_cancelled (self);
-    }
-}
-
-/**
- * gtd_manager_create_task_list:
- * @manager: a #GtdManager
- * @list: a #GtdTaskList
- *
- * Creates a new task list at the given source.
- */
-void
-gtd_manager_create_task_list (GtdManager  *self,
-                              GtdTaskList *list)
-{
-  g_autoptr (GError) error = NULL;
-  GtdProvider *provider;
-
-  g_return_if_fail (GTD_IS_MANAGER (self));
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
-
-  provider = gtd_task_list_get_provider (list);
-
-  gtd_provider_create_task_list (provider, list, self->cancellable, &error);
-
-  if (error)
-    {
-      g_warning ("Error creating task list: %s", error->message);
-      reset_cancellable_if_cancelled (self);
-    }
-}
-
-/**
- * gtd_manager_remove_task_list:
- * @manager: a #GtdManager
- * @list: a #GtdTaskList
- *
- * Deletes @list from the registry.
- */
-void
-gtd_manager_remove_task_list (GtdManager  *self,
-                              GtdTaskList *list)
-{
-  g_autoptr (GError) error = NULL;
-  GtdProvider *provider;
-
-  g_return_if_fail (GTD_IS_MANAGER (self));
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
-
-  provider = gtd_task_list_get_provider (list);
-
-  gtd_provider_remove_task_list (provider, list, self->cancellable, &error);
-
-  if (error)
-    {
-      g_warning ("Error removing task list: %s", error->message);
-      reset_cancellable_if_cancelled (self);
-    }
-
-  g_signal_emit (self,
-                 signals[LIST_REMOVED],
-                 0,
-                 list);
-}
-
-/**
- * gtd_manager_save_task_list:
- * @manager: a #GtdManager
- * @list: a #GtdTaskList
- *
- * Save or create @list.
- */
-void
-gtd_manager_save_task_list (GtdManager  *self,
-                            GtdTaskList *list)
-{
-  g_autoptr (GError) error = NULL;
-  GtdProvider *provider;
-
-  g_return_if_fail (GTD_IS_MANAGER (self));
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
-
-  provider = gtd_task_list_get_provider (list);
-
-  gtd_provider_update_task_list (provider, list, self->cancellable, &error);
-
-  if (error)
-    {
-      g_warning ("Error saving task list: %s", error->message);
-      reset_cancellable_if_cancelled (self);
-    }
-}
-
 /**
  * gtd_manager_get_task_lists:
  * @manager: a #GtdManager
@@ -1006,20 +796,32 @@ gtd_manager_set_is_first_run (GtdManager *self,
   g_settings_set_boolean (self->settings, "first-run", is_first_run);
 }
 
+/**
+ * gtd_manager_emit_error_message:
+ * @self: a #GtdManager
+ * @title: (nullable): the title of the error
+ * @description: (nullable): detailed description of the error
+ * @function: (nullable): function to be called when the notification is dismissed
+ * @user_data: user data
+ *
+ * Reports an error.
+ */
 void
-gtd_manager_emit_error_message (GtdManager                *self,
-                                const gchar               *primary_message,
-                                const gchar               *secondary_message,
-                                GtdNotificationActionFunc  function,
-                                gpointer                   user_data)
+gtd_manager_emit_error_message (GtdManager         *self,
+                                const gchar        *title,
+                                const gchar        *description,
+                                GtdErrorActionFunc  function,
+                                gpointer            user_data)
 {
   g_return_if_fail (GTD_IS_MANAGER (self));
 
-  emit_show_error_message (self,
-                           primary_message,
-                           secondary_message,
-                           function,
-                           user_data);
+  g_signal_emit (self,
+                 signals[SHOW_ERROR_MESSAGE],
+                 0,
+                 title,
+                 description,
+                 function,
+                 user_data);
 }
 
 /**
