@@ -16,15 +16,21 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "interfaces/gtd-activatable.h"
-#include "interfaces/gtd-panel.h"
-#include "interfaces/gtd-provider.h"
-#include "notification/gtd-notification.h"
+#ifndef GNOME_TODO_H
+#define GNOME_TODO_H
+
+#include "gtd-activatable.h"
 #include "gtd-enums.h"
 #include "gtd-manager.h"
+#include "gtd-notification.h"
 #include "gtd-object.h"
+#include "gtd-panel.h"
+#include "gtd-provider.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
 #include "gtd-task-list-view.h"
-#include "gtd-window.h"
 #include "gtd-types.h"
+#include "gtd-utils.h"
+#include "gtd-window.h"
+
+#endif /* GNOME_TODO_H */
