@@ -16,12 +16,14 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
+#define G_LOG_DOMAIN "GtdApplication"
+
 #include "config.h"
-#endif
 
 #include "gtd-application.h"
+#include "gtd-debug.h"
 #include "gtd-initial-setup-window.h"
+#include "gtd-log.h"
 #include "gtd-manager.h"
 #include "gtd-manager-protected.h"
 #include "gtd-plugin-dialog.h"
@@ -67,7 +69,9 @@ static void           gtd_application_quit                        (GSimpleAction
 G_DEFINE_TYPE (GtdApplication, gtd_application, GTK_TYPE_APPLICATION)
 
 static GOptionEntry cmd_options[] = {
-  { "quit", 'q', 0, G_OPTION_ARG_NONE, NULL, N_("Quit GNOME To Do"), NULL }
+  { "quit", 'q', 0, G_OPTION_ARG_NONE, NULL, N_("Quit GNOME To Do"), NULL },
+  { "debug", 'd', 0, G_OPTION_ARG_NONE, NULL, N_("Enable debug messages"), NULL },
+  { NULL }
 };
 
 static const GActionEntry gtd_application_entries[] = {
@@ -95,7 +99,7 @@ gtd_application_start_client (GSimpleAction *simple,
                               gpointer       user_data)
 {
   /* TODO */
-  g_message ("Starting up client");
+  g_debug ("Starting up client");
 }
 
 static void
@@ -173,8 +177,6 @@ gtd_application_quit (GSimpleAction *simple,
 GtdApplication *
 gtd_application_new (void)
 {
-  g_set_application_name (_("To Do"));
-
   return g_object_new (GTD_TYPE_APPLICATION,
                        "application-id", "org.gnome.Todo",
                        "flags", G_APPLICATION_HANDLES_COMMAND_LINE,
@@ -233,16 +235,14 @@ run_initial_setup (GtdApplication *application)
 static void
 gtd_application_activate (GApplication *application)
 {
+  GTD_ENTRY;
+
   /* FIXME: the initial setup is disabled for the 3.18 release because
    * we can't create tasklists on GOA accounts.
    */
   run_window (GTD_APPLICATION (application));
-}
 
-static void
-gtd_application_finalize (GObject *object)
-{
-  G_OBJECT_CLASS (gtd_application_parent_class)->finalize (object);
+  GTD_EXIT;
 }
 
 static void
@@ -253,6 +253,8 @@ gtd_application_startup (GApplication *application)
   g_autoptr (GFile) css_file;
   g_autofree gchar *theme_name, *theme_uri;
 
+  GTD_ENTRY;
+
   self = GTD_APPLICATION (application);
 
   /* add actions */
@@ -289,6 +291,8 @@ gtd_application_startup (GApplication *application)
 
   /* Load the plugins */
   gtd_manager_load_plugins (gtd_manager_get_default ());
+
+  GTD_EXIT;
 }
 
 static gint
@@ -322,18 +326,26 @@ gtd_application_local_command_line (GApplication   *application,
                                                                                  exit_status);
 }
 
+static gint
+gtd_application_handle_local_options (GApplication *application,
+                                      GVariantDict *options)
+{
+  if (g_variant_dict_contains (options, "debug"))
+    gtd_log_init ();
+
+  return -1;
+}
+
 static void
 gtd_application_class_init (GtdApplicationClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
-  object_class->finalize = gtd_application_finalize;
-
   application_class->activate = gtd_application_activate;
   application_class->startup = gtd_application_startup;
   application_class->command_line = gtd_application_command_line;
   application_class->local_command_line = gtd_application_local_command_line;
+  application_class->handle_local_options = gtd_application_handle_local_options;
 }
 
 static void
