@@ -30,12 +30,13 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (GtdEditPane, gtd_edit_pane, GTD, EDIT_PANE, GtkGrid)
 
-GtkWidget*                 gtd_edit_pane_new                 (void);
+GtkWidget*           gtd_edit_pane_new                           (void);
 
-GtdTask*                   gtd_edit_pane_get_task            (GtdEditPane        *pane);
+GtdTask*             gtd_edit_pane_get_task                      (GtdEditPane        *self);
+
+void                 gtd_edit_pane_set_task                      (GtdEditPane        *self,
+                                                                  GtdTask            *task);
 
-void                       gtd_edit_pane_set_task            (GtdEditPane        *pane,
-                                                              GtdTask            *task);
 G_END_DECLS
 
 #endif /* GTD_EDIT_PANE_H */
