@@ -62,9 +62,8 @@ struct _GtdTaskRow
   /* data */
   GtdTask            *task;
 
-  gint                destroy_row_timeout_id;
-  gint                update_title_timeout_id;
   gboolean            active;
+  gboolean            changed;
 };
 
 #define PRIORITY_ICON_SIZE 8
@@ -84,7 +83,7 @@ static void          on_priority_changed_cb                      (GtdTaskRow
                                                                   GParamSpec         *spec,
                                                                   GObject            *object);
 
-static void         on_task_title_changed_cb                     (GtdTaskRow         *self);
+static void          on_task_changed_cb                          (GtdTaskRow         *self);
 
 static void          on_toggle_active_cb                         (GtkWidget          *button,
                                                                   GtdTaskRow         *self);
@@ -225,21 +224,21 @@ gtd_task_row_set_task (GtdTaskRow *row,
     {
       gtk_label_set_label (row->task_list_label, gtd_task_list_get_name (gtd_task_get_list (task)));
 
-      g_signal_handlers_block_by_func (row->title_entry, on_task_title_changed_cb, row);
+      g_signal_handlers_block_by_func (row->title_entry, on_task_changed_cb, row);
       g_signal_handlers_block_by_func (row->done_check, on_complete_check_toggled_cb, row);
 
+      g_object_bind_property (task,
+                              "loading",
+                              row,
+                              "sensitive",
+                              G_BINDING_DEFAULT | G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
+
       g_object_bind_property (task,
                               "title",
                               row->title_entry,
                               "text",
                               G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
-      g_object_bind_property (task,
-                              "complete",
-                              row->done_check,
-                              "active",
-                              G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
-
       g_object_bind_property_full (task,
                                    "due-date",
                                    row->task_date_label,
@@ -273,7 +272,7 @@ gtd_task_row_set_task (GtdTaskRow *row,
                                 row);
 
       g_signal_handlers_unblock_by_func (row->done_check, on_complete_check_toggled_cb, row);
-      g_signal_handlers_block_by_func (row->title_entry, on_task_title_changed_cb, row);
+      g_signal_handlers_unblock_by_func (row->title_entry, on_task_changed_cb, row);
 
       gtd_edit_pane_set_task (GTD_EDIT_PANE (row->edit_panel), task);
     }
@@ -282,7 +281,6 @@ gtd_task_row_set_task (GtdTaskRow *row,
 }
 
 
-
 /*
  * Callbacks
  */
@@ -291,9 +289,6 @@ static void
 on_toggle_active_cb (GtkWidget  *button,
                      GtdTaskRow *self)
 {
-  if (self->active)
-    gtd_manager_update_task (gtd_manager_get_default (), self->task);
-
   gtd_task_row_set_active (self, !self->active);
 }
 
@@ -463,51 +458,38 @@ on_complete_changed_cb (GtdTaskRow *self,
                         GtdTask    *task)
 {
   GtkStyleContext *context;
+  gboolean complete;
 
+  GTD_ENTRY;
+
+  complete = gtd_task_get_complete (task);
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
 
-  if (gtd_task_get_complete (task))
+  if (complete)
     gtk_style_context_add_class (context, "complete");
   else
     gtk_style_context_remove_class (context, "complete");
-}
 
-static void
-on_toggle_complete_cb (GtkRevealer *revealer,
-                       GParamSpec  *pspec,
-                       GtdTaskRow  *self)
-{
-  g_signal_handlers_disconnect_by_func (revealer, on_toggle_complete_cb, self);
+  /* Update the toggle button as well */
+  g_signal_handlers_block_by_func (self->done_check, on_complete_check_toggled_cb, self);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->done_check), complete);
+  g_signal_handlers_unblock_by_func (self->done_check, on_complete_check_toggled_cb, self);
 
-  gtd_task_set_complete (self->task, !gtd_task_get_complete (self->task));
+  GTD_EXIT;
 }
 
 static void
 on_complete_check_toggled_cb (GtkToggleButton *button,
                               GtdTaskRow      *self)
 {
-  GtdTaskListView *listview;
+  GTD_ENTRY;
 
-  listview = GTD_TASK_LIST_VIEW (gtk_widget_get_ancestor (GTK_WIDGET (self), GTD_TYPE_TASK_LIST_VIEW));
+  g_assert (GTD_IS_TASK (self->task));
 
-  /*
-   * If the parent list view is showing completed tasks, we
-   * don't have to hide the row. Simply toggle the 'complete'
-   * property of the task.
-   */
-  if (gtd_task_list_view_get_show_completed (listview))
-    {
-      gtd_task_set_complete (self->task, !gtd_task_get_complete (self->task));
-      return;
-    }
+  gtd_task_set_complete (self->task, gtk_toggle_button_get_active (button));
+  gtd_provider_update_task (gtd_task_get_provider (self->task), self->task);
 
-  gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer),
-                                 !gtk_toggle_button_get_active (button));
-
-  g_signal_connect (self->revealer,
-                    "notify::child-revealed",
-                    G_CALLBACK (on_toggle_complete_cb),
-                    self);
+  GTD_EXIT;
 }
 
 static void
@@ -519,38 +501,12 @@ on_depth_changed_cb (GtdTaskRow *self,
                                self->handle_subtasks ? 32 * gtd_task_get_depth (task) + 3: 3);
 }
 
-static gboolean
-on_destroy_cb (GtkWidget *row)
-{
-  gtk_widget_destroy (row);
-
-  return G_SOURCE_REMOVE;
-}
-
-static gboolean
-on_update_title_timeout_cb (gpointer data)
-{
-  GtdTaskRow *self = data;
-
-  GTD_ENTRY;
-
-  gtd_manager_update_task (gtd_manager_get_default (), self->task);
-  self->update_title_timeout_id = 0;
-
-  GTD_RETURN (G_SOURCE_REMOVE);
-}
-
 static void
-on_task_title_changed_cb (GtdTaskRow *self)
+on_task_changed_cb (GtdTaskRow  *self)
 {
-  GTD_ENTRY;
+  g_debug ("Task changed");
 
-  if (self->update_title_timeout_id > 0)
-    g_source_remove (self->update_title_timeout_id);
-
-  self->update_title_timeout_id = g_timeout_add_seconds (2, on_update_title_timeout_cb, self);
-
-  GTD_EXIT;
+  self->changed = TRUE;
 }
 
 
@@ -559,8 +515,8 @@ on_task_title_changed_cb (GtdTaskRow *self)
  */
 
 static gboolean
-gtd_task_row__key_press_event (GtkWidget   *row,
-                               GdkEventKey *event)
+gtd_task_row_key_press_event (GtkWidget   *row,
+                              GdkEventKey *event)
 {
   GtdTaskRow *self = GTD_TASK_ROW (row);
 
@@ -584,11 +540,11 @@ gtd_task_row_finalize (GObject *object)
 {
   GtdTaskRow *self = GTD_TASK_ROW (object);
 
-  if (self->update_title_timeout_id > 0)
+  if (self->changed)
     {
       if (self->task)
-        gtd_manager_update_task (gtd_manager_get_default (), self->task);
-      self->update_title_timeout_id = 0;
+        gtd_provider_update_task (gtd_task_get_provider (self->task), self->task);
+      self->changed = FALSE;
     }
 
   g_clear_object (&self->task);
@@ -672,7 +628,7 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
   object_class->get_property = gtd_task_row_get_property;
   object_class->set_property = gtd_task_row_set_property;
 
-  widget_class->key_press_event = gtd_task_row__key_press_event;
+  widget_class->key_press_event = gtd_task_row_key_press_event;
   widget_class->get_preferred_width = gtd_row_get_preferred_width_with_max;
 
   g_type_ensure (GTD_TYPE_EXPANDABLE_ENTRY);
@@ -775,7 +731,7 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_mouse_over_event_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_mouse_over_dnd_event_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_remove_task_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_task_title_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_task_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_toggle_active_cb);
 
   gtk_widget_class_set_css_name (widget_class, "taskrow");
@@ -886,21 +842,8 @@ gtd_task_row_destroy (GtdTaskRow *self)
 {
   g_return_if_fail (GTD_IS_TASK_ROW (self));
 
-  if (!gtk_revealer_get_child_revealed (self->revealer))
-    {
-      gtk_widget_destroy (GTK_WIDGET (self));
-    }
-  else if (self->destroy_row_timeout_id == 0)
-    {
-      guint duration;
-
-      duration = gtk_revealer_get_transition_duration (self->revealer);
-
-      gtk_revealer_set_reveal_child (self->revealer, FALSE);
-      self->destroy_row_timeout_id = g_timeout_add (duration,
-                                                    (GSourceFunc) on_destroy_cb,
-                                                    self);
-    }
+  gtk_revealer_set_reveal_child (self->revealer, FALSE);
+  g_signal_connect_swapped (self->revealer, "notify::child-revealed", G_CALLBACK (gtk_widget_destroy), self);
 }
 
 gboolean
@@ -974,6 +917,15 @@ gtd_task_row_set_active (GtdTaskRow *self,
   /* And the listbox */
   gtk_revealer_set_reveal_child (GTK_REVEALER (self->edit_panel_revealer), active);
 
+  /* Save the task if it is not being loaded */
+  if (!active && !gtd_object_get_loading (GTD_OBJECT (self->task)) && self->changed)
+    {
+      g_debug ("Saving task…");
+
+      gtd_provider_update_task (gtd_task_get_provider (self->task), self->task);
+      self->changed = FALSE;
+    }
+
   g_signal_emit (self, active ? signals[ENTER] : signals[EXIT], 0);
 }
 
