@@ -48,6 +48,7 @@ typedef struct
   gchar           *title;
 
   gint32           priority;
+  gint64           position;
   gboolean         complete;
 } GtdTaskPrivate;
 
@@ -63,6 +64,7 @@ enum
   PROP_DUE_DATE,
   PROP_LIST,
   PROP_PARENT,
+  PROP_POSITION,
   PROP_PRIORITY,
   PROP_TITLE,
   LAST_PROP
@@ -282,7 +284,19 @@ gtd_task_real_set_complete (GtdTask  *self,
 static GDateTime*
 gtd_task_real_get_creation_date (GtdTask *self)
 {
-  return NULL;
+  GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+
+  return priv->creation_date ? g_date_time_ref (priv->creation_date) : NULL;
+}
+
+static void
+gtd_task_real_set_creation_date (GtdTask   *self,
+                                 GDateTime *dt)
+{
+  GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+
+  g_clear_pointer (&priv->creation_date, g_date_time_unref);
+  priv->creation_date = dt ? g_date_time_ref (dt) : NULL;
 }
 
 static const gchar*
@@ -323,6 +337,23 @@ gtd_task_real_set_due_date (GtdTask   *self,
     priv->due_date = g_date_time_ref (due_date);
 }
 
+static gint64
+gtd_task_real_get_position (GtdTask *self)
+{
+  GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+
+  return priv->position;
+}
+
+static void
+gtd_task_real_set_position (GtdTask *self,
+                            gint64   position)
+{
+  GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+
+  priv->position = position;
+}
+
 static gint32
 gtd_task_real_get_priority (GtdTask *self)
 {
@@ -420,6 +451,10 @@ gtd_task_get_property (GObject    *object,
       g_value_set_object (value, priv->parent);
       break;
 
+    case PROP_POSITION:
+      g_value_set_int64 (value, gtd_task_get_position (self));
+      break;
+
     case PROP_PRIORITY:
       g_value_set_int (value, gtd_task_get_priority (self));
       break;
@@ -447,6 +482,10 @@ gtd_task_set_property (GObject      *object,
       gtd_task_set_complete (self, g_value_get_boolean (value));
       break;
 
+    case PROP_CREATION_DATE:
+      gtd_task_set_creation_date (self, g_value_get_boxed (value));
+      break;
+
     case PROP_DESCRIPTION:
       gtd_task_set_description (self, g_value_get_string (value));
       break;
@@ -459,6 +498,10 @@ gtd_task_set_property (GObject      *object,
       gtd_task_set_list (self, g_value_get_object (value));
       break;
 
+    case PROP_POSITION:
+      gtd_task_set_position (self, g_value_get_int64 (value));
+      break;
+
     case PROP_PRIORITY:
       gtd_task_set_priority (self, g_value_get_int (value));
       break;
@@ -480,10 +523,13 @@ gtd_task_class_init (GtdTaskClass *klass)
   klass->get_complete = gtd_task_real_get_complete;
   klass->set_complete = gtd_task_real_set_complete;
   klass->get_creation_date = gtd_task_real_get_creation_date;
+  klass->set_creation_date = gtd_task_real_set_creation_date;
   klass->get_description = gtd_task_real_get_description;
   klass->set_description = gtd_task_real_set_description;
   klass->get_due_date = gtd_task_real_get_due_date;
   klass->set_due_date = gtd_task_real_set_due_date;
+  klass->get_position = gtd_task_real_get_position;
+  klass->set_position = gtd_task_real_set_position;
   klass->get_priority = gtd_task_real_get_priority;
   klass->set_priority = gtd_task_real_set_priority;
   klass->get_title = gtd_task_real_get_title;
@@ -522,7 +568,7 @@ gtd_task_class_init (GtdTaskClass *klass)
                             "Creation date of the task",
                             "The day the task was created.",
                             G_TYPE_DATE_TIME,
-                            G_PARAM_READABLE));
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
    * GtdTask::depth:
@@ -613,6 +659,22 @@ gtd_task_class_init (GtdTaskClass *klass)
                           0,
                           G_PARAM_READWRITE));
 
+  /**
+   * GtdTask::position:
+   *
+   * Position of the task, -1 if not set.
+   */
+  g_object_class_install_property (
+        object_class,
+        PROP_POSITION,
+        g_param_spec_int64 ("position",
+                            "Position of the task",
+                            "The position of the task. -1 means no position, and tasks will be sorted alphabetically.",
+                            -1,
+                            G_MAXINT64,
+                            0,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GtdTask::title:
    *
@@ -664,6 +726,9 @@ gtd_task_class_init (GtdTaskClass *klass)
 static void
 gtd_task_init (GtdTask *self)
 {
+  GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+
+  priv->position = -1;
 }
 
 /**
@@ -706,7 +771,7 @@ void
 gtd_task_set_complete (GtdTask  *task,
                        gboolean  complete)
 {
-  g_assert (GTD_IS_TASK (task));
+  g_return_if_fail (GTD_IS_TASK (task));
 
   if (gtd_task_get_complete (task) == complete)
     return;
@@ -735,6 +800,26 @@ gtd_task_get_creation_date (GtdTask *task)
   return GTD_TASK_CLASS (G_OBJECT_GET_CLASS (task))->get_creation_date (task);
 }
 
+/**
+ * gtd_task_set_creation_date:
+ * @task: a #GtdTask
+ *
+ * Sets the creation date of @task.
+ */
+void
+gtd_task_set_creation_date (GtdTask   *task,
+                            GDateTime *dt)
+{
+  g_return_if_fail (GTD_IS_TASK (task));
+
+  if (gtd_task_get_creation_date (task) == dt)
+    return;
+
+  GTD_TASK_CLASS (G_OBJECT_GET_CLASS (task))->set_creation_date (task, dt);
+
+  g_object_notify (G_OBJECT (task), "complete");
+}
+
 /**
  * gtd_task_get_description:
  * @task: a #GtdTask
@@ -874,6 +959,46 @@ gtd_task_set_list (GtdTask     *task,
   g_object_notify (G_OBJECT (task), "list");
 }
 
+/**
+ * gtd_task_get_position:
+ * @task: a #GtdTask
+ *
+ * Returns the position of @task inside the parent #GtdTaskList,
+ * or -1 if not set.
+ *
+ * Returns: the position of the task, or -1
+ */
+gint64
+gtd_task_get_position (GtdTask *self)
+{
+  g_return_val_if_fail (GTD_IS_TASK (self), -1);
+
+  return GTD_TASK_CLASS (G_OBJECT_GET_CLASS (self))->get_position (self);
+}
+
+/**
+ * gtd_task_set_position:
+ * @task: a #GtdTask
+ * @position: the priority of @task, or -1
+ *
+ * Sets the @task position inside the parent #GtdTaskList. It
+ * is up to the interface to handle two or more #GtdTask with
+ * the same position value.
+ */
+void
+gtd_task_set_position (GtdTask *self,
+                       gint64   position)
+{
+  g_return_if_fail (GTD_IS_TASK (self));
+
+  if (gtd_task_get_position (self) == position)
+    return;
+
+  GTD_TASK_CLASS (G_OBJECT_GET_CLASS (self))->set_position (self, position);
+
+  g_object_notify (G_OBJECT (self), "position");
+}
+
 /**
  * gtd_task_get_priority:
  * @task: a #GtdTask
@@ -1003,6 +1128,18 @@ gtd_task_compare (GtdTask *t1,
   if (retval != 0)
     return retval;
 
+  /*
+   * The custom position overrides any comparison we can make. To keep compatibility,
+   * for now, we only compare by position if both tasks have a custom position set.
+   */
+  if (gtd_task_get_position (t1) != -1 && gtd_task_get_position (t2) != -1)
+    {
+      retval = gtd_task_get_position (t1) - gtd_task_get_position (t2);
+
+      if (retval != 0)
+        return retval;
+    }
+
   /*
    * First, compare by ::complete.
    */
@@ -1143,9 +1280,7 @@ gtd_task_add_subtask (GtdTask *self,
   priv = gtd_task_get_instance_private (self);
 
   if (!g_list_find (priv->subtasks, subtask) && !gtd_task_is_subtask (subtask, self))
-    {
-      g_signal_emit (self, signals[SUBTASK_ADDED], 0, subtask);
-    }
+    g_signal_emit (self, signals[SUBTASK_ADDED], 0, subtask);
 }
 
 /**
@@ -1167,9 +1302,7 @@ gtd_task_remove_subtask (GtdTask *self,
   priv = gtd_task_get_instance_private (self);
 
   if (g_list_find (priv->subtasks, subtask))
-    {
-      g_signal_emit (self, signals[SUBTASK_REMOVED], 0, subtask);
-    }
+    g_signal_emit (self, signals[SUBTASK_REMOVED], 0, subtask);
 }
 
 /**
@@ -1186,16 +1319,17 @@ gtd_task_is_subtask (GtdTask *self,
                      GtdTask *subtask)
 {
   GtdTask *aux;
-  GQueue *queue;
+  GQueue queue;
   gboolean is_subtask;
 
   g_return_val_if_fail (GTD_IS_TASK (self), FALSE);
   g_return_val_if_fail (GTD_IS_TASK (subtask), FALSE);
 
   aux = self;
-  queue = g_queue_new ();
   is_subtask = FALSE;
 
+  g_queue_init (&queue);
+
   do
     {
       GtdTaskPrivate *priv;
@@ -1212,18 +1346,16 @@ gtd_task_is_subtask (GtdTask *self,
               break;
             }
 
-          g_queue_push_tail (queue, l->data);
+          g_queue_push_tail (&queue, l->data);
         }
 
       if (is_subtask)
         break;
 
-      aux = g_queue_pop_head (queue);
+      aux = g_queue_pop_head (&queue);
     }
   while (aux);
 
-  g_queue_free (queue);
-
   return is_subtask;
 }
 
@@ -1246,3 +1378,35 @@ gtd_task_get_depth (GtdTask *self)
 
   return priv->depth;
 }
+
+/**
+ * gtd_task_get_provider:
+ * @self: a #GtdTaskList
+ *
+ * Utility function to retrieve the data provider that backs this
+ * task. Notice that this is exactly the same as writting:
+ *
+ * |[<!-- language="C" -->
+ * GtdTaskList *list;
+ * GtdProvider *provider;
+ *
+ * list = gtd_task_get_list (task);
+ * provider = gtd_task_list_get_provider (list);
+ * ]|
+ *
+ * Returns: (transfer none)(nullable): the #GtdProvider of this task's list.
+ */
+GtdProvider*
+gtd_task_get_provider (GtdTask *self)
+{
+  GtdTaskPrivate *priv;
+
+  g_return_val_if_fail (GTD_IS_TASK (self), NULL);
+
+  priv = gtd_task_get_instance_private (self);
+
+  if (priv->list)
+    return gtd_task_list_get_provider (priv->list);
+
+  return NULL;
+}
