@@ -38,6 +38,8 @@ struct _GtdTaskClass
                                                        gboolean              complete);
 
   GDateTime*    (*get_creation_date)                  (GtdTask              *self);
+  void          (*set_creation_date)                  (GtdTask              *self,
+                                                       GDateTime            *dt);
 
   const gchar*  (*get_description)                    (GtdTask              *self);
   void          (*set_description)                    (GtdTask              *self,
@@ -47,6 +49,10 @@ struct _GtdTaskClass
   void          (*set_due_date)                       (GtdTask              *self,
                                                        GDateTime            *due_date);
 
+  gint64        (*get_position)                       (GtdTask              *task);
+  void          (*set_position)                       (GtdTask              *task,
+                                                       gint64                position);
+
   gint32        (*get_priority)                       (GtdTask              *self);
   void          (*set_priority)                       (GtdTask              *self,
                                                        gint32                priority);
@@ -75,6 +81,9 @@ void                gtd_task_set_complete             (GtdTask              *tas
 
 GDateTime*          gtd_task_get_creation_date        (GtdTask              *task);
 
+void                gtd_task_set_creation_date        (GtdTask              *task,
+                                                       GDateTime            *dt);
+
 const gchar*        gtd_task_get_description          (GtdTask              *task);
 
 void                gtd_task_set_description          (GtdTask              *task,
@@ -90,6 +99,11 @@ GtdTaskList*        gtd_task_get_list                 (GtdTask              *tas
 void                gtd_task_set_list                 (GtdTask              *task,
                                                        GtdTaskList          *list);
 
+gint64              gtd_task_get_position             (GtdTask              *self);
+
+void                gtd_task_set_position             (GtdTask              *self,
+                                                       gint64                position);
+
 gint                gtd_task_get_priority             (GtdTask              *task);
 
 void                gtd_task_set_priority             (GtdTask              *task,
@@ -118,6 +132,8 @@ gboolean            gtd_task_is_subtask               (GtdTask              *sel
 
 gint                gtd_task_get_depth                (GtdTask              *self);
 
+GtdProvider*        gtd_task_get_provider             (GtdTask              *self);
+
 G_END_DECLS
 
 #endif /* GTD_TASK_H */
