@@ -49,29 +49,9 @@ enum
 static guint signals[NUM_SIGNALS] = { 0, };
 
 
-/*
- * Default implementations
- */
-
-static GtdTask*
-gtd_provider_default_generate_task (GtdProvider *self)
-{
-  g_autofree gchar *uuid;
-  GtdTask *task;
-
-  uuid = g_uuid_string_random ();
-
-  task = gtd_task_new ();
-  gtd_object_set_uid (GTD_OBJECT (task), uuid);
-
-  return task;
-}
-
 static void
 gtd_provider_default_init (GtdProviderInterface *iface)
 {
-  iface->generate_task = gtd_provider_default_generate_task;
-
   /**
    * GtdProvider::enabled:
    *
@@ -295,126 +275,104 @@ gtd_provider_get_icon (GtdProvider *provider)
  * gtd_provider_create_task:
  * @provider: a #GtdProvider
  * @task: a #GtdTask
- * @cancellable: (nullable): a #GCancellable
- * @error: (nullable): return location for a #GError
  *
  * Creates the given task in @provider.
  */
 void
-gtd_provider_create_task (GtdProvider   *provider,
-                          GtdTask       *task,
-                          GCancellable  *cancellable,
-                          GError       **error)
+gtd_provider_create_task (GtdProvider *provider,
+                          GtdTaskList *list,
+                          const gchar *title,
+                          GDateTime   *due_date)
 {
   g_return_if_fail (GTD_IS_PROVIDER (provider));
   g_return_if_fail (GTD_PROVIDER_GET_IFACE (provider)->create_task);
 
-  GTD_PROVIDER_GET_IFACE (provider)->create_task (provider, task, cancellable, error);
+  GTD_PROVIDER_GET_IFACE (provider)->create_task (provider, list, title, due_date);
 }
 
 /**
  * gtd_provider_update_task:
  * @provider: a #GtdProvider
  * @task: a #GtdTask
- * @cancellable: (nullable): a #GCancellable
- * @error: (nullable): return location for a #GError
  *
  * Updates the given task in @provider.
  */
 void
-gtd_provider_update_task (GtdProvider   *provider,
-                          GtdTask       *task,
-                          GCancellable  *cancellable,
-                          GError       **error)
+gtd_provider_update_task (GtdProvider *provider,
+                          GtdTask     *task)
 {
   g_return_if_fail (GTD_IS_PROVIDER (provider));
   g_return_if_fail (GTD_PROVIDER_GET_IFACE (provider)->update_task);
 
-  GTD_PROVIDER_GET_IFACE (provider)->update_task (provider, task, cancellable, error);
+  GTD_PROVIDER_GET_IFACE (provider)->update_task (provider, task);
 }
 
 /**
  * gtd_provider_remove_task:
  * @provider: a #GtdProvider
  * @task: a #GtdTask
- * @cancellable: (nullable): a #GCancellable
- * @error: (nullable): return location for a #GError
  *
  * Removes the given task from @provider.
  */
 void
-gtd_provider_remove_task (GtdProvider   *provider,
-                          GtdTask       *task,
-                          GCancellable  *cancellable,
-                          GError       **error)
+gtd_provider_remove_task (GtdProvider *provider,
+                          GtdTask     *task)
 {
   g_return_if_fail (GTD_IS_PROVIDER (provider));
   g_return_if_fail (GTD_PROVIDER_GET_IFACE (provider)->remove_task);
 
-  GTD_PROVIDER_GET_IFACE (provider)->remove_task (provider, task, cancellable, error);
+  GTD_PROVIDER_GET_IFACE (provider)->remove_task (provider, task);
 }
 
 /**
  * gtd_provider_create_task_list:
  * @provider: a #GtdProvider
- * @list: a #GtdTaskList
- * @cancellable: (nullable): a #GCancellable
- * @error: (nullable): return location for a #GError
+ * @name: (nullable): the name of the new task list
  *
  * Creates the given list in @provider.
  */
 void
-gtd_provider_create_task_list (GtdProvider   *provider,
-                               GtdTaskList   *list,
-                               GCancellable  *cancellable,
-                               GError       **error)
+gtd_provider_create_task_list (GtdProvider *provider,
+                               const gchar *name)
 {
   g_return_if_fail (GTD_IS_PROVIDER (provider));
   g_return_if_fail (GTD_PROVIDER_GET_IFACE (provider)->create_task_list);
 
-  GTD_PROVIDER_GET_IFACE (provider)->create_task_list (provider, list, cancellable, error);
+  GTD_PROVIDER_GET_IFACE (provider)->create_task_list (provider, name);
 }
 
 /**
  * gtd_provider_update_task_list:
  * @provider: a #GtdProvider
  * @list: a #GtdTaskList
- * @cancellable: (nullable): a #GCancellable
- * @error: (nullable): return location for a #GError
  *
  * Updates the given list in @provider.
  */
 void
-gtd_provider_update_task_list (GtdProvider   *provider,
-                               GtdTaskList   *list,
-                               GCancellable  *cancellable,
-                               GError       **error)
+gtd_provider_update_task_list (GtdProvider *provider,
+                               GtdTaskList *list)
 {
   g_return_if_fail (GTD_IS_PROVIDER (provider));
   g_return_if_fail (GTD_PROVIDER_GET_IFACE (provider)->update_task_list);
 
-  GTD_PROVIDER_GET_IFACE (provider)->update_task_list (provider, list, cancellable, error);
+  GTD_PROVIDER_GET_IFACE (provider)->update_task_list (provider, list);
 }
 
 /**
  * gtd_provider_remove_task_list:
  * @provider: a #GtdProvider
  * @list: a #GtdTaskList
- * @cancellable: (nullable): a #GCancellable
- * @error: (nullable): return location for a #GError
  *
  * Removes the given list from @provider.
  */
 void
-gtd_provider_remove_task_list (GtdProvider   *provider,
-                               GtdTaskList   *list,
-                               GCancellable  *cancellable,
-                               GError       **error)
+gtd_provider_remove_task_list (GtdProvider *provider,
+                               GtdTaskList *list)
 {
   g_return_if_fail (GTD_IS_PROVIDER (provider));
   g_return_if_fail (GTD_PROVIDER_GET_IFACE (provider)->remove_task_list);
 
-  GTD_PROVIDER_GET_IFACE (provider)->remove_task_list (provider, list, cancellable, error);
+  GTD_PROVIDER_GET_IFACE (provider)->remove_task_list (provider, list);
 }
 
 /**
@@ -469,20 +427,3 @@ gtd_provider_set_default_task_list (GtdProvider *provider,
 
   return GTD_PROVIDER_GET_IFACE (provider)->set_default_task_list (provider, list);
 }
-
-/**
- * gtd_provider_generate_task:
- * @provider: a #GtdProvider
- *
- * Creates a new, empty #GtdTask.
- *
- * Returns: (transfer full): a #GtdTask
- */
-GtdTask*
-gtd_provider_generate_task (GtdProvider *self)
-{
-  g_return_val_if_fail (GTD_IS_PROVIDER (self), NULL);
-  g_return_val_if_fail (GTD_PROVIDER_GET_IFACE (self)->generate_task, NULL);
-
-  return GTD_PROVIDER_GET_IFACE (self)->generate_task (self);
-}
