@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GtdProviderSelector"
+
 #include "gtd-application.h"
 #include "gtd-manager.h"
 #include "interfaces/gtd-provider.h"
@@ -89,7 +91,7 @@ display_header_func (GtkListBoxRow *row,
 }
 
 static void
-gtd_provider_selector__default_provider_changed (GtdProviderSelector *selector)
+default_provider_changed_cb (GtdProviderSelector *selector)
 {
   GtdProvider *current;
   GtdManager *manager;
@@ -215,8 +217,8 @@ gtd_provider_selector__check_toggled (GtdProviderSelector *selector,
 }
 
 static void
-gtd_provider_selector__remove_provider (GtdProviderSelector *selector,
-                                        GtdProvider         *provider)
+remove_provider (GtdProviderSelector *selector,
+                 GtdProvider         *provider)
 {
   GList *children;
   GList *l;
@@ -264,8 +266,8 @@ gtd_provider_selector__remove_provider (GtdProviderSelector *selector,
 }
 
 static void
-gtd_provider_selector__add_provider (GtdProviderSelector *selector,
-                                     GtdProvider         *provider)
+add_provider (GtdProviderSelector *selector,
+              GtdProvider         *provider)
 {
   GtkWidget *row;
   const gchar *provider_id;
@@ -276,10 +278,12 @@ gtd_provider_selector__add_provider (GtdProviderSelector *selector,
   row = gtd_provider_row_new (provider);
   provider_id = gtd_provider_get_id (provider);
 
+  g_debug ("Adding provider %s", provider_id);
+
   gtk_container_add (GTK_CONTAINER (selector->listbox), row);
 
   /* track the local provider row */
-  if (g_strcmp0 (provider_id, "local") == 0)
+  if (g_str_has_prefix (provider_id, "local") == 0)
     {
       gtk_widget_set_visible (row, selector->show_local_provider);
       selector->local_row = row;
@@ -287,18 +291,17 @@ gtd_provider_selector__add_provider (GtdProviderSelector *selector,
 
   /* Auto selects the default provider row when needed */
   if (selector->select_default &&
-      //gtd_provider_get_is_default (provider) &&
       !gtd_provider_selector_get_selected_provider (selector))
     {
       gtd_provider_selector_set_selected_provider (selector, provider);
     }
 
   /* hide the related stub row */
-  if (g_strcmp0 (provider_id, "exchange") == 0)
+  if (g_str_has_prefix (provider_id, "exchange") == 0)
     gtk_widget_hide (selector->exchange_stub_row);
-  else if (g_strcmp0 (provider_id, "google") == 0)
+  else if (g_str_has_prefix (provider_id, "google") == 0)
     gtk_widget_hide (selector->google_stub_row);
-  else if (g_strcmp0 (provider_id, "owncloud") == 0)
+  else if (g_str_has_prefix (provider_id, "owncloud") == 0)
     gtk_widget_hide (selector->owncloud_stub_row);
 }
 
@@ -317,7 +320,7 @@ gtd_provider_selector__fill_accounts (GtdProviderSelector *selector)
   providers = gtd_manager_get_providers (manager);
 
   for (l = providers; l != NULL; l = l->next)
-    gtd_provider_selector__add_provider (selector, l->data);
+    add_provider (selector, l->data);
 
   g_list_free (providers);
 }
@@ -516,17 +519,17 @@ gtd_provider_selector_init (GtdProviderSelector *self)
 
   g_signal_connect_swapped (manager,
                             "notify::default-provider",
-                            G_CALLBACK (gtd_provider_selector__default_provider_changed),
+                            G_CALLBACK (default_provider_changed_cb),
                             self);
 
   g_signal_connect_swapped (manager,
                             "provider-added",
-                            G_CALLBACK (gtd_provider_selector__add_provider),
+                            G_CALLBACK (add_provider),
                             self);
 
   g_signal_connect_swapped (manager,
                             "provider-removed",
-                            G_CALLBACK (gtd_provider_selector__remove_provider),
+                            G_CALLBACK (remove_provider),
                             self);
 
 }
@@ -686,11 +689,11 @@ gtd_provider_selector_set_selected_provider (GtdProviderSelector *selector,
 
   for (l = children; l != NULL; l = l->next)
     {
-      if (GTD_IS_PROVIDER_ROW (l->data))
-        {
-          gtd_provider_row_set_selected (l->data, gtd_provider_row_get_provider (l->data) == provider);
-          g_signal_emit (selector, signals[PROVIDER_SELECTED], 0, provider);
-        }
+      if (!GTD_IS_PROVIDER_ROW (l->data))
+        continue;
+
+      gtd_provider_row_set_selected (l->data, gtd_provider_row_get_provider (l->data) == provider);
+      g_signal_emit (selector, signals[PROVIDER_SELECTED], 0, provider);
     }
 
   g_list_free (children);
@@ -723,7 +726,7 @@ gtd_provider_selector_get_show_stub_rows (GtdProviderSelector *selector)
  */
 void
 gtd_provider_selector_set_show_stub_rows (GtdProviderSelector *selector,
-                                         gboolean            show_stub_rows)
+                                          gboolean            show_stub_rows)
 {
   g_return_if_fail (GTD_IS_PROVIDER_SELECTOR (selector));
 
